#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: zun/criapi/api.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from zun.criapi import gogo_pb2 as zun_dot_criapi_dot_gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='zun/criapi/api.proto',
  package='runtime.v1alpha2',
  syntax='proto3',
  serialized_options=_b('Z\010v1alpha2\330\341\036\000\200\342\036\001\310\341\036\001\310\342\036\001\340\342\036\001\320\342\036\001\220\343\036\000'),
  serialized_pb=_b('\n\x14zun/criapi/api.proto\x12\x10runtime.v1alpha2\x1a\x15zun/criapi/gogo.proto\"!\n\x0eVersionRequest\x12\x0f\n\x07version\x18\x01 \x01(\t\"n\n\x0fVersionResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\x12\x14\n\x0cruntime_name\x18\x02 \x01(\t\x12\x17\n\x0fruntime_version\x18\x03 \x01(\t\x12\x1b\n\x13runtime_api_version\x18\x04 \x01(\t\"?\n\tDNSConfig\x12\x0f\n\x07servers\x18\x01 \x03(\t\x12\x10\n\x08searches\x18\x02 \x03(\t\x12\x0f\n\x07options\x18\x03 \x03(\t\"w\n\x0bPortMapping\x12,\n\x08protocol\x18\x01 \x01(\x0e\x32\x1a.runtime.v1alpha2.Protocol\x12\x16\n\x0e\x63ontainer_port\x18\x02 \x01(\x05\x12\x11\n\thost_port\x18\x03 \x01(\x05\x12\x0f\n\x07host_ip\x18\x04 \x01(\t\"\x96\x01\n\x05Mount\x12\x16\n\x0e\x63ontainer_path\x18\x01 \x01(\t\x12\x11\n\thost_path\x18\x02 \x01(\t\x12\x10\n\x08readonly\x18\x03 \x01(\x08\x12\x17\n\x0fselinux_relabel\x18\x04 \x01(\x08\x12\x37\n\x0bpropagation\x18\x05 \x01(\x0e\x32\".runtime.v1alpha2.MountPropagation\"\x9f\x01\n\x0fNamespaceOption\x12\x30\n\x07network\x18\x01 \x01(\x0e\x32\x1f.runtime.v1alpha2.NamespaceMode\x12,\n\x03pid\x18\x02 \x01(\x0e\x32\x1f.runtime.v1alpha2.NamespaceMode\x12,\n\x03ipc\x18\x03 \x01(\x0e\x32\x1f.runtime.v1alpha2.NamespaceMode\"\x1b\n\nInt64Value\x12\r\n\x05value\x18\x01 \x01(\x03\"\xe4\x02\n\x1bLinuxSandboxSecurityContext\x12<\n\x11namespace_options\x18\x01 \x01(\x0b\x32!.runtime.v1alpha2.NamespaceOption\x12\x38\n\x0fselinux_options\x18\x02 \x01(\x0b\x32\x1f.runtime.v1alpha2.SELinuxOption\x12\x31\n\x0brun_as_user\x18\x03 \x01(\x0b\x32\x1c.runtime.v1alpha2.Int64Value\x12\x32\n\x0crun_as_group\x18\x08 \x01(\x0b\x32\x1c.runtime.v1alpha2.Int64Value\x12\x17\n\x0freadonly_rootfs\x18\x04 \x01(\x08\x12\x1b\n\x13supplemental_groups\x18\x05 \x03(\x03\x12\x12\n\nprivileged\x18\x06 \x01(\x08\x12\x1c\n\x14seccomp_profile_path\x18\x07 \x01(\t\"\xee\x01\n\x15LinuxPodSandboxConfig\x12\x15\n\rcgroup_parent\x18\x01 \x01(\t\x12G\n\x10security_context\x18\x02 \x01(\x0b\x32-.runtime.v1alpha2.LinuxSandboxSecurityContext\x12\x45\n\x07sysctls\x18\x03 \x03(\x0b\x32\x34.runtime.v1alpha2.LinuxPodSandboxConfig.SysctlsEntry\x1a.\n\x0cSysctlsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"S\n\x12PodSandboxMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03uid\x18\x02 \x01(\t\x12\x11\n\tnamespace\x18\x03 \x01(\t\x12\x0f\n\x07\x61ttempt\x18\x04 \x01(\r\"\xff\x03\n\x10PodSandboxConfig\x12\x36\n\x08metadata\x18\x01 \x01(\x0b\x32$.runtime.v1alpha2.PodSandboxMetadata\x12\x10\n\x08hostname\x18\x02 \x01(\t\x12\x15\n\rlog_directory\x18\x03 \x01(\t\x12/\n\ndns_config\x18\x04 \x01(\x0b\x32\x1b.runtime.v1alpha2.DNSConfig\x12\x34\n\rport_mappings\x18\x05 \x03(\x0b\x32\x1d.runtime.v1alpha2.PortMapping\x12>\n\x06labels\x18\x06 \x03(\x0b\x32..runtime.v1alpha2.PodSandboxConfig.LabelsEntry\x12H\n\x0b\x61nnotations\x18\x07 \x03(\x0b\x32\x33.runtime.v1alpha2.PodSandboxConfig.AnnotationsEntry\x12\x36\n\x05linux\x18\x08 \x01(\x0b\x32\'.runtime.v1alpha2.LinuxPodSandboxConfig\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"c\n\x14RunPodSandboxRequest\x12\x32\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\".runtime.v1alpha2.PodSandboxConfig\x12\x17\n\x0fruntime_handler\x18\x02 \x01(\t\"/\n\x15RunPodSandboxResponse\x12\x16\n\x0epod_sandbox_id\x18\x01 \x01(\t\"/\n\x15StopPodSandboxRequest\x12\x16\n\x0epod_sandbox_id\x18\x01 \x01(\t\"\x18\n\x16StopPodSandboxResponse\"1\n\x17RemovePodSandboxRequest\x12\x16\n\x0epod_sandbox_id\x18\x01 \x01(\t\"\x1a\n\x18RemovePodSandboxResponse\"B\n\x17PodSandboxStatusRequest\x12\x16\n\x0epod_sandbox_id\x18\x01 \x01(\t\x12\x0f\n\x07verbose\x18\x02 \x01(\x08\"\x13\n\x05PodIP\x12\n\n\x02ip\x18\x01 \x01(\t\"V\n\x17PodSandboxNetworkStatus\x12\n\n\x02ip\x18\x01 \x01(\t\x12/\n\x0e\x61\x64\x64itional_ips\x18\x02 \x03(\x0b\x32\x17.runtime.v1alpha2.PodIP\"?\n\tNamespace\x12\x32\n\x07options\x18\x02 \x01(\x0b\x32!.runtime.v1alpha2.NamespaceOption\"H\n\x15LinuxPodSandboxStatus\x12/\n\nnamespaces\x18\x01 \x01(\x0b\x32\x1b.runtime.v1alpha2.Namespace\"\x96\x04\n\x10PodSandboxStatus\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x08metadata\x18\x02 \x01(\x0b\x32$.runtime.v1alpha2.PodSandboxMetadata\x12\x30\n\x05state\x18\x03 \x01(\x0e\x32!.runtime.v1alpha2.PodSandboxState\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12:\n\x07network\x18\x05 \x01(\x0b\x32).runtime.v1alpha2.PodSandboxNetworkStatus\x12\x36\n\x05linux\x18\x06 \x01(\x0b\x32\'.runtime.v1alpha2.LinuxPodSandboxStatus\x12>\n\x06labels\x18\x07 \x03(\x0b\x32..runtime.v1alpha2.PodSandboxStatus.LabelsEntry\x12H\n\x0b\x61nnotations\x18\x08 \x03(\x0b\x32\x33.runtime.v1alpha2.PodSandboxStatus.AnnotationsEntry\x12\x17\n\x0fruntime_handler\x18\t \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xbf\x01\n\x18PodSandboxStatusResponse\x12\x32\n\x06status\x18\x01 \x01(\x0b\x32\".runtime.v1alpha2.PodSandboxStatus\x12\x42\n\x04info\x18\x02 \x03(\x0b\x32\x34.runtime.v1alpha2.PodSandboxStatusResponse.InfoEntry\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"H\n\x14PodSandboxStateValue\x12\x30\n\x05state\x18\x01 \x01(\x0e\x32!.runtime.v1alpha2.PodSandboxState\"\xda\x01\n\x10PodSandboxFilter\x12\n\n\x02id\x18\x01 \x01(\t\x12\x35\n\x05state\x18\x02 \x01(\x0b\x32&.runtime.v1alpha2.PodSandboxStateValue\x12M\n\x0elabel_selector\x18\x03 \x03(\x0b\x32\x35.runtime.v1alpha2.PodSandboxFilter.LabelSelectorEntry\x1a\x34\n\x12LabelSelectorEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"K\n\x15ListPodSandboxRequest\x12\x32\n\x06\x66ilter\x18\x01 \x01(\x0b\x32\".runtime.v1alpha2.PodSandboxFilter\"\x90\x03\n\nPodSandbox\x12\n\n\x02id\x18\x01 \x01(\t\x12\x36\n\x08metadata\x18\x02 \x01(\x0b\x32$.runtime.v1alpha2.PodSandboxMetadata\x12\x30\n\x05state\x18\x03 \x01(\x0e\x32!.runtime.v1alpha2.PodSandboxState\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12\x38\n\x06labels\x18\x05 \x03(\x0b\x32(.runtime.v1alpha2.PodSandbox.LabelsEntry\x12\x42\n\x0b\x61nnotations\x18\x06 \x03(\x0b\x32-.runtime.v1alpha2.PodSandbox.AnnotationsEntry\x12\x17\n\x0fruntime_handler\x18\x07 \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"E\n\x16ListPodSandboxResponse\x12+\n\x05items\x18\x01 \x03(\x0b\x32\x1c.runtime.v1alpha2.PodSandbox\"\x1a\n\tImageSpec\x12\r\n\x05image\x18\x01 \x01(\t\"&\n\x08KeyValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xee\x01\n\x17LinuxContainerResources\x12\x12\n\ncpu_period\x18\x01 \x01(\x03\x12\x11\n\tcpu_quota\x18\x02 \x01(\x03\x12\x12\n\ncpu_shares\x18\x03 \x01(\x03\x12\x1d\n\x15memory_limit_in_bytes\x18\x04 \x01(\x03\x12\x15\n\room_score_adj\x18\x05 \x01(\x03\x12\x13\n\x0b\x63puset_cpus\x18\x06 \x01(\t\x12\x13\n\x0b\x63puset_mems\x18\x07 \x01(\t\x12\x38\n\x0fhugepage_limits\x18\x08 \x03(\x0b\x32\x1f.runtime.v1alpha2.HugepageLimit\"1\n\rHugepageLimit\x12\x11\n\tpage_size\x18\x01 \x01(\t\x12\r\n\x05limit\x18\x02 \x01(\x04\"H\n\rSELinuxOption\x12\x0c\n\x04user\x18\x01 \x01(\t\x12\x0c\n\x04role\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\r\n\x05level\x18\x04 \x01(\t\"A\n\nCapability\x12\x18\n\x10\x61\x64\x64_capabilities\x18\x01 \x03(\t\x12\x19\n\x11\x64rop_capabilities\x18\x02 \x03(\t\"\x91\x04\n\x1dLinuxContainerSecurityContext\x12\x32\n\x0c\x63\x61pabilities\x18\x01 \x01(\x0b\x32\x1c.runtime.v1alpha2.Capability\x12\x12\n\nprivileged\x18\x02 \x01(\x08\x12<\n\x11namespace_options\x18\x03 \x01(\x0b\x32!.runtime.v1alpha2.NamespaceOption\x12\x38\n\x0fselinux_options\x18\x04 \x01(\x0b\x32\x1f.runtime.v1alpha2.SELinuxOption\x12\x31\n\x0brun_as_user\x18\x05 \x01(\x0b\x32\x1c.runtime.v1alpha2.Int64Value\x12\x32\n\x0crun_as_group\x18\x0c \x01(\x0b\x32\x1c.runtime.v1alpha2.Int64Value\x12\x17\n\x0frun_as_username\x18\x06 \x01(\t\x12\x17\n\x0freadonly_rootfs\x18\x07 \x01(\x08\x12\x1b\n\x13supplemental_groups\x18\x08 \x03(\x03\x12\x18\n\x10\x61pparmor_profile\x18\t \x01(\t\x12\x1c\n\x14seccomp_profile_path\x18\n \x01(\t\x12\x14\n\x0cno_new_privs\x18\x0b \x01(\x08\x12\x14\n\x0cmasked_paths\x18\r \x03(\t\x12\x16\n\x0ereadonly_paths\x18\x0e \x03(\t\"\x9f\x01\n\x14LinuxContainerConfig\x12<\n\tresources\x18\x01 \x01(\x0b\x32).runtime.v1alpha2.LinuxContainerResources\x12I\n\x10security_context\x18\x02 \x01(\x0b\x32/.runtime.v1alpha2.LinuxContainerSecurityContext\"S\n\x1fWindowsContainerSecurityContext\x12\x17\n\x0frun_as_username\x18\x01 \x01(\t\x12\x17\n\x0f\x63redential_spec\x18\x02 \x01(\t\"\xa5\x01\n\x16WindowsContainerConfig\x12>\n\tresources\x18\x01 \x01(\x0b\x32+.runtime.v1alpha2.WindowsContainerResources\x12K\n\x10security_context\x18\x02 \x01(\x0b\x32\x31.runtime.v1alpha2.WindowsContainerSecurityContext\"v\n\x19WindowsContainerResources\x12\x12\n\ncpu_shares\x18\x01 \x01(\x03\x12\x11\n\tcpu_count\x18\x02 \x01(\x03\x12\x13\n\x0b\x63pu_maximum\x18\x03 \x01(\x03\x12\x1d\n\x15memory_limit_in_bytes\x18\x04 \x01(\x03\"2\n\x11\x43ontainerMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07\x61ttempt\x18\x02 \x01(\r\"H\n\x06\x44\x65vice\x12\x16\n\x0e\x63ontainer_path\x18\x01 \x01(\t\x12\x11\n\thost_path\x18\x02 \x01(\t\x12\x13\n\x0bpermissions\x18\x03 \x01(\t\"\xc5\x05\n\x0f\x43ontainerConfig\x12\x35\n\x08metadata\x18\x01 \x01(\x0b\x32#.runtime.v1alpha2.ContainerMetadata\x12*\n\x05image\x18\x02 \x01(\x0b\x32\x1b.runtime.v1alpha2.ImageSpec\x12\x0f\n\x07\x63ommand\x18\x03 \x03(\t\x12\x0c\n\x04\x61rgs\x18\x04 \x03(\t\x12\x13\n\x0bworking_dir\x18\x05 \x01(\t\x12(\n\x04\x65nvs\x18\x06 \x03(\x0b\x32\x1a.runtime.v1alpha2.KeyValue\x12\'\n\x06mounts\x18\x07 \x03(\x0b\x32\x17.runtime.v1alpha2.Mount\x12)\n\x07\x64\x65vices\x18\x08 \x03(\x0b\x32\x18.runtime.v1alpha2.Device\x12=\n\x06labels\x18\t \x03(\x0b\x32-.runtime.v1alpha2.ContainerConfig.LabelsEntry\x12G\n\x0b\x61nnotations\x18\n \x03(\x0b\x32\x32.runtime.v1alpha2.ContainerConfig.AnnotationsEntry\x12\x10\n\x08log_path\x18\x0b \x01(\t\x12\r\n\x05stdin\x18\x0c \x01(\x08\x12\x12\n\nstdin_once\x18\r \x01(\x08\x12\x0b\n\x03tty\x18\x0e \x01(\x08\x12\x35\n\x05linux\x18\x0f \x01(\x0b\x32&.runtime.v1alpha2.LinuxContainerConfig\x12\x39\n\x07windows\x18\x10 \x01(\x0b\x32(.runtime.v1alpha2.WindowsContainerConfig\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9f\x01\n\x16\x43reateContainerRequest\x12\x16\n\x0epod_sandbox_id\x18\x01 \x01(\t\x12\x31\n\x06\x63onfig\x18\x02 \x01(\x0b\x32!.runtime.v1alpha2.ContainerConfig\x12:\n\x0esandbox_config\x18\x03 \x01(\x0b\x32\".runtime.v1alpha2.PodSandboxConfig\"/\n\x17\x43reateContainerResponse\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\"-\n\x15StartContainerRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\"\x18\n\x16StartContainerResponse\"=\n\x14StopContainerRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\x0f\n\x07timeout\x18\x02 \x01(\x03\"\x17\n\x15StopContainerResponse\".\n\x16RemoveContainerRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\"\x19\n\x17RemoveContainerResponse\"F\n\x13\x43ontainerStateValue\x12/\n\x05state\x18\x01 \x01(\x0e\x32 .runtime.v1alpha2.ContainerState\"\xef\x01\n\x0f\x43ontainerFilter\x12\n\n\x02id\x18\x01 \x01(\t\x12\x34\n\x05state\x18\x02 \x01(\x0b\x32%.runtime.v1alpha2.ContainerStateValue\x12\x16\n\x0epod_sandbox_id\x18\x03 \x01(\t\x12L\n\x0elabel_selector\x18\x04 \x03(\x0b\x32\x34.runtime.v1alpha2.ContainerFilter.LabelSelectorEntry\x1a\x34\n\x12LabelSelectorEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"J\n\x15ListContainersRequest\x12\x31\n\x06\x66ilter\x18\x01 \x01(\x0b\x32!.runtime.v1alpha2.ContainerFilter\"\xc9\x03\n\tContainer\x12\n\n\x02id\x18\x01 \x01(\t\x12\x16\n\x0epod_sandbox_id\x18\x02 \x01(\t\x12\x35\n\x08metadata\x18\x03 \x01(\x0b\x32#.runtime.v1alpha2.ContainerMetadata\x12*\n\x05image\x18\x04 \x01(\x0b\x32\x1b.runtime.v1alpha2.ImageSpec\x12\x11\n\timage_ref\x18\x05 \x01(\t\x12/\n\x05state\x18\x06 \x01(\x0e\x32 .runtime.v1alpha2.ContainerState\x12\x12\n\ncreated_at\x18\x07 \x01(\x03\x12\x37\n\x06labels\x18\x08 \x03(\x0b\x32\'.runtime.v1alpha2.Container.LabelsEntry\x12\x41\n\x0b\x61nnotations\x18\t \x03(\x0b\x32,.runtime.v1alpha2.Container.AnnotationsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"I\n\x16ListContainersResponse\x12/\n\ncontainers\x18\x01 \x03(\x0b\x32\x1b.runtime.v1alpha2.Container\"?\n\x16\x43ontainerStatusRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\x0f\n\x07verbose\x18\x02 \x01(\x08\"\xdb\x04\n\x0f\x43ontainerStatus\x12\n\n\x02id\x18\x01 \x01(\t\x12\x35\n\x08metadata\x18\x02 \x01(\x0b\x32#.runtime.v1alpha2.ContainerMetadata\x12/\n\x05state\x18\x03 \x01(\x0e\x32 .runtime.v1alpha2.ContainerState\x12\x12\n\ncreated_at\x18\x04 \x01(\x03\x12\x12\n\nstarted_at\x18\x05 \x01(\x03\x12\x13\n\x0b\x66inished_at\x18\x06 \x01(\x03\x12\x11\n\texit_code\x18\x07 \x01(\x05\x12*\n\x05image\x18\x08 \x01(\x0b\x32\x1b.runtime.v1alpha2.ImageSpec\x12\x11\n\timage_ref\x18\t \x01(\t\x12\x0e\n\x06reason\x18\n \x01(\t\x12\x0f\n\x07message\x18\x0b \x01(\t\x12=\n\x06labels\x18\x0c \x03(\x0b\x32-.runtime.v1alpha2.ContainerStatus.LabelsEntry\x12G\n\x0b\x61nnotations\x18\r \x03(\x0b\x32\x32.runtime.v1alpha2.ContainerStatus.AnnotationsEntry\x12\'\n\x06mounts\x18\x0e \x03(\x0b\x32\x17.runtime.v1alpha2.Mount\x12\x10\n\x08log_path\x18\x0f \x01(\t\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xbc\x01\n\x17\x43ontainerStatusResponse\x12\x31\n\x06status\x18\x01 \x01(\x0b\x32!.runtime.v1alpha2.ContainerStatus\x12\x41\n\x04info\x18\x02 \x03(\x0b\x32\x33.runtime.v1alpha2.ContainerStatusResponse.InfoEntry\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"q\n\x1fUpdateContainerResourcesRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\x38\n\x05linux\x18\x02 \x01(\x0b\x32).runtime.v1alpha2.LinuxContainerResources\"\"\n UpdateContainerResourcesResponse\"E\n\x0f\x45xecSyncRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x03(\t\x12\x0f\n\x07timeout\x18\x03 \x01(\x03\"E\n\x10\x45xecSyncResponse\x12\x0e\n\x06stdout\x18\x01 \x01(\x0c\x12\x0e\n\x06stderr\x18\x02 \x01(\x0c\x12\x11\n\texit_code\x18\x03 \x01(\x05\"l\n\x0b\x45xecRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x03(\t\x12\x0b\n\x03tty\x18\x03 \x01(\x08\x12\r\n\x05stdin\x18\x04 \x01(\x08\x12\x0e\n\x06stdout\x18\x05 \x01(\x08\x12\x0e\n\x06stderr\x18\x06 \x01(\x08\"\x1b\n\x0c\x45xecResponse\x12\x0b\n\x03url\x18\x01 \x01(\t\"a\n\rAttachRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\x12\r\n\x05stdin\x18\x02 \x01(\x08\x12\x0b\n\x03tty\x18\x03 \x01(\x08\x12\x0e\n\x06stdout\x18\x04 \x01(\x08\x12\x0e\n\x06stderr\x18\x05 \x01(\x08\"\x1d\n\x0e\x41ttachResponse\x12\x0b\n\x03url\x18\x01 \x01(\t\":\n\x12PortForwardRequest\x12\x16\n\x0epod_sandbox_id\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x03(\x05\"\"\n\x13PortForwardResponse\x12\x0b\n\x03url\x18\x01 \x01(\t\"9\n\x0bImageFilter\x12*\n\x05image\x18\x01 \x01(\x0b\x32\x1b.runtime.v1alpha2.ImageSpec\"B\n\x11ListImagesRequest\x12-\n\x06\x66ilter\x18\x01 \x01(\x0b\x32\x1d.runtime.v1alpha2.ImageFilter\"\x87\x01\n\x05Image\x12\n\n\x02id\x18\x01 \x01(\t\x12\x11\n\trepo_tags\x18\x02 \x03(\t\x12\x14\n\x0crepo_digests\x18\x03 \x03(\t\x12\x0c\n\x04size\x18\x04 \x01(\x04\x12)\n\x03uid\x18\x05 \x01(\x0b\x32\x1c.runtime.v1alpha2.Int64Value\x12\x10\n\x08username\x18\x06 \x01(\t\"=\n\x12ListImagesResponse\x12\'\n\x06images\x18\x01 \x03(\x0b\x32\x17.runtime.v1alpha2.Image\"Q\n\x12ImageStatusRequest\x12*\n\x05image\x18\x01 \x01(\x0b\x32\x1b.runtime.v1alpha2.ImageSpec\x12\x0f\n\x07verbose\x18\x02 \x01(\x08\"\xa9\x01\n\x13ImageStatusResponse\x12&\n\x05image\x18\x01 \x01(\x0b\x32\x17.runtime.v1alpha2.Image\x12=\n\x04info\x18\x02 \x03(\x0b\x32/.runtime.v1alpha2.ImageStatusResponse.InfoEntry\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x86\x01\n\nAuthConfig\x12\x10\n\x08username\x18\x01 \x01(\t\x12\x10\n\x08password\x18\x02 \x01(\t\x12\x0c\n\x04\x61uth\x18\x03 \x01(\t\x12\x16\n\x0eserver_address\x18\x04 \x01(\t\x12\x16\n\x0eidentity_token\x18\x05 \x01(\t\x12\x16\n\x0eregistry_token\x18\x06 \x01(\t\"\xa6\x01\n\x10PullImageRequest\x12*\n\x05image\x18\x01 \x01(\x0b\x32\x1b.runtime.v1alpha2.ImageSpec\x12*\n\x04\x61uth\x18\x02 \x01(\x0b\x32\x1c.runtime.v1alpha2.AuthConfig\x12:\n\x0esandbox_config\x18\x03 \x01(\x0b\x32\".runtime.v1alpha2.PodSandboxConfig\"&\n\x11PullImageResponse\x12\x11\n\timage_ref\x18\x01 \x01(\t\"@\n\x12RemoveImageRequest\x12*\n\x05image\x18\x01 \x01(\x0b\x32\x1b.runtime.v1alpha2.ImageSpec\"\x15\n\x13RemoveImageResponse\"!\n\rNetworkConfig\x12\x10\n\x08pod_cidr\x18\x01 \x01(\t\"H\n\rRuntimeConfig\x12\x37\n\x0enetwork_config\x18\x01 \x01(\x0b\x32\x1f.runtime.v1alpha2.NetworkConfig\"U\n\x1aUpdateRuntimeConfigRequest\x12\x37\n\x0eruntime_config\x18\x01 \x01(\x0b\x32\x1f.runtime.v1alpha2.RuntimeConfig\"\x1d\n\x1bUpdateRuntimeConfigResponse\"Q\n\x10RuntimeCondition\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0e\n\x06status\x18\x02 \x01(\x08\x12\x0e\n\x06reason\x18\x03 \x01(\t\x12\x0f\n\x07message\x18\x04 \x01(\t\"G\n\rRuntimeStatus\x12\x36\n\nconditions\x18\x01 \x03(\x0b\x32\".runtime.v1alpha2.RuntimeCondition\" \n\rStatusRequest\x12\x0f\n\x07verbose\x18\x01 \x01(\x08\"\xa8\x01\n\x0eStatusResponse\x12/\n\x06status\x18\x01 \x01(\x0b\x32\x1f.runtime.v1alpha2.RuntimeStatus\x12\x38\n\x04info\x18\x02 \x03(\x0b\x32*.runtime.v1alpha2.StatusResponse.InfoEntry\x1a+\n\tInfoEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x14\n\x12ImageFsInfoRequest\"\x1c\n\x0bUInt64Value\x12\r\n\x05value\x18\x01 \x01(\x04\"*\n\x14\x46ilesystemIdentifier\x12\x12\n\nmountpoint\x18\x01 \x01(\t\"\xc2\x01\n\x0f\x46ilesystemUsage\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\x35\n\x05\x66s_id\x18\x02 \x01(\x0b\x32&.runtime.v1alpha2.FilesystemIdentifier\x12\x31\n\nused_bytes\x18\x03 \x01(\x0b\x32\x1d.runtime.v1alpha2.UInt64Value\x12\x32\n\x0binodes_used\x18\x04 \x01(\x0b\x32\x1d.runtime.v1alpha2.UInt64Value\"S\n\x13ImageFsInfoResponse\x12<\n\x11image_filesystems\x18\x01 \x03(\x0b\x32!.runtime.v1alpha2.FilesystemUsage\"-\n\x15\x43ontainerStatsRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\"I\n\x16\x43ontainerStatsResponse\x12/\n\x05stats\x18\x01 \x01(\x0b\x32 .runtime.v1alpha2.ContainerStats\"S\n\x19ListContainerStatsRequest\x12\x36\n\x06\x66ilter\x18\x01 \x01(\x0b\x32&.runtime.v1alpha2.ContainerStatsFilter\"\xc3\x01\n\x14\x43ontainerStatsFilter\x12\n\n\x02id\x18\x01 \x01(\t\x12\x16\n\x0epod_sandbox_id\x18\x02 \x01(\t\x12Q\n\x0elabel_selector\x18\x03 \x03(\x0b\x32\x39.runtime.v1alpha2.ContainerStatsFilter.LabelSelectorEntry\x1a\x34\n\x12LabelSelectorEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"M\n\x1aListContainerStatsResponse\x12/\n\x05stats\x18\x01 \x03(\x0b\x32 .runtime.v1alpha2.ContainerStats\"\xcb\x02\n\x13\x43ontainerAttributes\x12\n\n\x02id\x18\x01 \x01(\t\x12\x35\n\x08metadata\x18\x02 \x01(\x0b\x32#.runtime.v1alpha2.ContainerMetadata\x12\x41\n\x06labels\x18\x03 \x03(\x0b\x32\x31.runtime.v1alpha2.ContainerAttributes.LabelsEntry\x12K\n\x0b\x61nnotations\x18\x04 \x03(\x0b\x32\x36.runtime.v1alpha2.ContainerAttributes.AnnotationsEntry\x1a-\n\x0bLabelsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x32\n\x10\x41nnotationsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xde\x01\n\x0e\x43ontainerStats\x12\x39\n\nattributes\x18\x01 \x01(\x0b\x32%.runtime.v1alpha2.ContainerAttributes\x12\'\n\x03\x63pu\x18\x02 \x01(\x0b\x32\x1a.runtime.v1alpha2.CpuUsage\x12-\n\x06memory\x18\x03 \x01(\x0b\x32\x1d.runtime.v1alpha2.MemoryUsage\x12\x39\n\x0ewritable_layer\x18\x04 \x01(\x0b\x32!.runtime.v1alpha2.FilesystemUsage\"]\n\x08\x43puUsage\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12>\n\x17usage_core_nano_seconds\x18\x02 \x01(\x0b\x32\x1d.runtime.v1alpha2.UInt64Value\"Z\n\x0bMemoryUsage\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\x38\n\x11working_set_bytes\x18\x02 \x01(\x0b\x32\x1d.runtime.v1alpha2.UInt64Value\"1\n\x19ReopenContainerLogRequest\x12\x14\n\x0c\x63ontainer_id\x18\x01 \x01(\t\"\x1c\n\x1aReopenContainerLogResponse*&\n\x08Protocol\x12\x07\n\x03TCP\x10\x00\x12\x07\n\x03UDP\x10\x01\x12\x08\n\x04SCTP\x10\x02*m\n\x10MountPropagation\x12\x17\n\x13PROPAGATION_PRIVATE\x10\x00\x12!\n\x1dPROPAGATION_HOST_TO_CONTAINER\x10\x01\x12\x1d\n\x19PROPAGATION_BIDIRECTIONAL\x10\x02*1\n\rNamespaceMode\x12\x07\n\x03POD\x10\x00\x12\r\n\tCONTAINER\x10\x01\x12\x08\n\x04NODE\x10\x02*:\n\x0fPodSandboxState\x12\x11\n\rSANDBOX_READY\x10\x00\x12\x14\n\x10SANDBOX_NOTREADY\x10\x01*k\n\x0e\x43ontainerState\x12\x15\n\x11\x43ONTAINER_CREATED\x10\x00\x12\x15\n\x11\x43ONTAINER_RUNNING\x10\x01\x12\x14\n\x10\x43ONTAINER_EXITED\x10\x02\x12\x15\n\x11\x43ONTAINER_UNKNOWN\x10\x03\x32\xc1\x11\n\x0eRuntimeService\x12P\n\x07Version\x12 .runtime.v1alpha2.VersionRequest\x1a!.runtime.v1alpha2.VersionResponse\"\x00\x12\x62\n\rRunPodSandbox\x12&.runtime.v1alpha2.RunPodSandboxRequest\x1a\'.runtime.v1alpha2.RunPodSandboxResponse\"\x00\x12\x65\n\x0eStopPodSandbox\x12\'.runtime.v1alpha2.StopPodSandboxRequest\x1a(.runtime.v1alpha2.StopPodSandboxResponse\"\x00\x12k\n\x10RemovePodSandbox\x12).runtime.v1alpha2.RemovePodSandboxRequest\x1a*.runtime.v1alpha2.RemovePodSandboxResponse\"\x00\x12k\n\x10PodSandboxStatus\x12).runtime.v1alpha2.PodSandboxStatusRequest\x1a*.runtime.v1alpha2.PodSandboxStatusResponse\"\x00\x12\x65\n\x0eListPodSandbox\x12\'.runtime.v1alpha2.ListPodSandboxRequest\x1a(.runtime.v1alpha2.ListPodSandboxResponse\"\x00\x12h\n\x0f\x43reateContainer\x12(.runtime.v1alpha2.CreateContainerRequest\x1a).runtime.v1alpha2.CreateContainerResponse\"\x00\x12\x65\n\x0eStartContainer\x12\'.runtime.v1alpha2.StartContainerRequest\x1a(.runtime.v1alpha2.StartContainerResponse\"\x00\x12\x62\n\rStopContainer\x12&.runtime.v1alpha2.StopContainerRequest\x1a\'.runtime.v1alpha2.StopContainerResponse\"\x00\x12h\n\x0fRemoveContainer\x12(.runtime.v1alpha2.RemoveContainerRequest\x1a).runtime.v1alpha2.RemoveContainerResponse\"\x00\x12\x65\n\x0eListContainers\x12\'.runtime.v1alpha2.ListContainersRequest\x1a(.runtime.v1alpha2.ListContainersResponse\"\x00\x12h\n\x0f\x43ontainerStatus\x12(.runtime.v1alpha2.ContainerStatusRequest\x1a).runtime.v1alpha2.ContainerStatusResponse\"\x00\x12\x83\x01\n\x18UpdateContainerResources\x12\x31.runtime.v1alpha2.UpdateContainerResourcesRequest\x1a\x32.runtime.v1alpha2.UpdateContainerResourcesResponse\"\x00\x12q\n\x12ReopenContainerLog\x12+.runtime.v1alpha2.ReopenContainerLogRequest\x1a,.runtime.v1alpha2.ReopenContainerLogResponse\"\x00\x12S\n\x08\x45xecSync\x12!.runtime.v1alpha2.ExecSyncRequest\x1a\".runtime.v1alpha2.ExecSyncResponse\"\x00\x12G\n\x04\x45xec\x12\x1d.runtime.v1alpha2.ExecRequest\x1a\x1e.runtime.v1alpha2.ExecResponse\"\x00\x12M\n\x06\x41ttach\x12\x1f.runtime.v1alpha2.AttachRequest\x1a .runtime.v1alpha2.AttachResponse\"\x00\x12\\\n\x0bPortForward\x12$.runtime.v1alpha2.PortForwardRequest\x1a%.runtime.v1alpha2.PortForwardResponse\"\x00\x12\x65\n\x0e\x43ontainerStats\x12\'.runtime.v1alpha2.ContainerStatsRequest\x1a(.runtime.v1alpha2.ContainerStatsResponse\"\x00\x12q\n\x12ListContainerStats\x12+.runtime.v1alpha2.ListContainerStatsRequest\x1a,.runtime.v1alpha2.ListContainerStatsResponse\"\x00\x12t\n\x13UpdateRuntimeConfig\x12,.runtime.v1alpha2.UpdateRuntimeConfigRequest\x1a-.runtime.v1alpha2.UpdateRuntimeConfigResponse\"\x00\x12M\n\x06Status\x12\x1f.runtime.v1alpha2.StatusRequest\x1a .runtime.v1alpha2.StatusResponse\"\x00\x32\xdb\x03\n\x0cImageService\x12Y\n\nListImages\x12#.runtime.v1alpha2.ListImagesRequest\x1a$.runtime.v1alpha2.ListImagesResponse\"\x00\x12\\\n\x0bImageStatus\x12$.runtime.v1alpha2.ImageStatusRequest\x1a%.runtime.v1alpha2.ImageStatusResponse\"\x00\x12V\n\tPullImage\x12\".runtime.v1alpha2.PullImageRequest\x1a#.runtime.v1alpha2.PullImageResponse\"\x00\x12\\\n\x0bRemoveImage\x12$.runtime.v1alpha2.RemoveImageRequest\x1a%.runtime.v1alpha2.RemoveImageResponse\"\x00\x12\\\n\x0bImageFsInfo\x12$.runtime.v1alpha2.ImageFsInfoRequest\x1a%.runtime.v1alpha2.ImageFsInfoResponse\"\x00\x42&Z\x08v1alpha2\xd8\xe1\x1e\x00\x80\xe2\x1e\x01\xc8\xe1\x1e\x01\xc8\xe2\x1e\x01\xe0\xe2\x1e\x01\xd0\xe2\x1e\x01\x90\xe3\x1e\x00\x62\x06proto3')
  ,
  dependencies=[zun_dot_criapi_dot_gogo__pb2.DESCRIPTOR,])

_PROTOCOL = _descriptor.EnumDescriptor(
  name='Protocol',
  full_name='runtime.v1alpha2.Protocol',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='TCP', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UDP', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SCTP', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12731,
  serialized_end=12769,
)
_sym_db.RegisterEnumDescriptor(_PROTOCOL)

Protocol = enum_type_wrapper.EnumTypeWrapper(_PROTOCOL)
_MOUNTPROPAGATION = _descriptor.EnumDescriptor(
  name='MountPropagation',
  full_name='runtime.v1alpha2.MountPropagation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PROPAGATION_PRIVATE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROPAGATION_HOST_TO_CONTAINER', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROPAGATION_BIDIRECTIONAL', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12771,
  serialized_end=12880,
)
_sym_db.RegisterEnumDescriptor(_MOUNTPROPAGATION)

MountPropagation = enum_type_wrapper.EnumTypeWrapper(_MOUNTPROPAGATION)
_NAMESPACEMODE = _descriptor.EnumDescriptor(
  name='NamespaceMode',
  full_name='runtime.v1alpha2.NamespaceMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='POD', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONTAINER', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='NODE', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12882,
  serialized_end=12931,
)
_sym_db.RegisterEnumDescriptor(_NAMESPACEMODE)

NamespaceMode = enum_type_wrapper.EnumTypeWrapper(_NAMESPACEMODE)
_PODSANDBOXSTATE = _descriptor.EnumDescriptor(
  name='PodSandboxState',
  full_name='runtime.v1alpha2.PodSandboxState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SANDBOX_READY', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SANDBOX_NOTREADY', index=1, number=1,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12933,
  serialized_end=12991,
)
_sym_db.RegisterEnumDescriptor(_PODSANDBOXSTATE)

PodSandboxState = enum_type_wrapper.EnumTypeWrapper(_PODSANDBOXSTATE)
_CONTAINERSTATE = _descriptor.EnumDescriptor(
  name='ContainerState',
  full_name='runtime.v1alpha2.ContainerState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='CONTAINER_CREATED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONTAINER_RUNNING', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONTAINER_EXITED', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CONTAINER_UNKNOWN', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=12993,
  serialized_end=13100,
)
_sym_db.RegisterEnumDescriptor(_CONTAINERSTATE)

ContainerState = enum_type_wrapper.EnumTypeWrapper(_CONTAINERSTATE)
TCP = 0
UDP = 1
SCTP = 2
PROPAGATION_PRIVATE = 0
PROPAGATION_HOST_TO_CONTAINER = 1
PROPAGATION_BIDIRECTIONAL = 2
POD = 0
CONTAINER = 1
NODE = 2
SANDBOX_READY = 0
SANDBOX_NOTREADY = 1
CONTAINER_CREATED = 0
CONTAINER_RUNNING = 1
CONTAINER_EXITED = 2
CONTAINER_UNKNOWN = 3



_VERSIONREQUEST = _descriptor.Descriptor(
  name='VersionRequest',
  full_name='runtime.v1alpha2.VersionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='runtime.v1alpha2.VersionRequest.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=98,
)


_VERSIONRESPONSE = _descriptor.Descriptor(
  name='VersionResponse',
  full_name='runtime.v1alpha2.VersionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='runtime.v1alpha2.VersionResponse.version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runtime_name', full_name='runtime.v1alpha2.VersionResponse.runtime_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runtime_version', full_name='runtime.v1alpha2.VersionResponse.runtime_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runtime_api_version', full_name='runtime.v1alpha2.VersionResponse.runtime_api_version', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=100,
  serialized_end=210,
)


_DNSCONFIG = _descriptor.Descriptor(
  name='DNSConfig',
  full_name='runtime.v1alpha2.DNSConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='servers', full_name='runtime.v1alpha2.DNSConfig.servers', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='searches', full_name='runtime.v1alpha2.DNSConfig.searches', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='options', full_name='runtime.v1alpha2.DNSConfig.options', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=212,
  serialized_end=275,
)


_PORTMAPPING = _descriptor.Descriptor(
  name='PortMapping',
  full_name='runtime.v1alpha2.PortMapping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='protocol', full_name='runtime.v1alpha2.PortMapping.protocol', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='container_port', full_name='runtime.v1alpha2.PortMapping.container_port', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host_port', full_name='runtime.v1alpha2.PortMapping.host_port', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host_ip', full_name='runtime.v1alpha2.PortMapping.host_ip', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=277,
  serialized_end=396,
)


_MOUNT = _descriptor.Descriptor(
  name='Mount',
  full_name='runtime.v1alpha2.Mount',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_path', full_name='runtime.v1alpha2.Mount.container_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host_path', full_name='runtime.v1alpha2.Mount.host_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='readonly', full_name='runtime.v1alpha2.Mount.readonly', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selinux_relabel', full_name='runtime.v1alpha2.Mount.selinux_relabel', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='propagation', full_name='runtime.v1alpha2.Mount.propagation', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=399,
  serialized_end=549,
)


_NAMESPACEOPTION = _descriptor.Descriptor(
  name='NamespaceOption',
  full_name='runtime.v1alpha2.NamespaceOption',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='network', full_name='runtime.v1alpha2.NamespaceOption.network', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pid', full_name='runtime.v1alpha2.NamespaceOption.pid', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ipc', full_name='runtime.v1alpha2.NamespaceOption.ipc', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=552,
  serialized_end=711,
)


_INT64VALUE = _descriptor.Descriptor(
  name='Int64Value',
  full_name='runtime.v1alpha2.Int64Value',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.Int64Value.value', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=713,
  serialized_end=740,
)


_LINUXSANDBOXSECURITYCONTEXT = _descriptor.Descriptor(
  name='LinuxSandboxSecurityContext',
  full_name='runtime.v1alpha2.LinuxSandboxSecurityContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace_options', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.namespace_options', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selinux_options', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.selinux_options', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_as_user', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_user', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_as_group', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.run_as_group', index=3,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='readonly_rootfs', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.readonly_rootfs', index=4,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='supplemental_groups', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.supplemental_groups', index=5,
      number=5, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='privileged', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.privileged', index=6,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seccomp_profile_path', full_name='runtime.v1alpha2.LinuxSandboxSecurityContext.seccomp_profile_path', index=7,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=743,
  serialized_end=1099,
)


_LINUXPODSANDBOXCONFIG_SYSCTLSENTRY = _descriptor.Descriptor(
  name='SysctlsEntry',
  full_name='runtime.v1alpha2.LinuxPodSandboxConfig.SysctlsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.LinuxPodSandboxConfig.SysctlsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.LinuxPodSandboxConfig.SysctlsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1294,
  serialized_end=1340,
)

_LINUXPODSANDBOXCONFIG = _descriptor.Descriptor(
  name='LinuxPodSandboxConfig',
  full_name='runtime.v1alpha2.LinuxPodSandboxConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cgroup_parent', full_name='runtime.v1alpha2.LinuxPodSandboxConfig.cgroup_parent', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='security_context', full_name='runtime.v1alpha2.LinuxPodSandboxConfig.security_context', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sysctls', full_name='runtime.v1alpha2.LinuxPodSandboxConfig.sysctls', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_LINUXPODSANDBOXCONFIG_SYSCTLSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1102,
  serialized_end=1340,
)


_PODSANDBOXMETADATA = _descriptor.Descriptor(
  name='PodSandboxMetadata',
  full_name='runtime.v1alpha2.PodSandboxMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='runtime.v1alpha2.PodSandboxMetadata.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uid', full_name='runtime.v1alpha2.PodSandboxMetadata.uid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace', full_name='runtime.v1alpha2.PodSandboxMetadata.namespace', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attempt', full_name='runtime.v1alpha2.PodSandboxMetadata.attempt', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1342,
  serialized_end=1425,
)


_PODSANDBOXCONFIG_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='runtime.v1alpha2.PodSandboxConfig.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandboxConfig.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandboxConfig.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1842,
  serialized_end=1887,
)

_PODSANDBOXCONFIG_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='runtime.v1alpha2.PodSandboxConfig.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandboxConfig.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandboxConfig.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1889,
  serialized_end=1939,
)

_PODSANDBOXCONFIG = _descriptor.Descriptor(
  name='PodSandboxConfig',
  full_name='runtime.v1alpha2.PodSandboxConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='runtime.v1alpha2.PodSandboxConfig.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hostname', full_name='runtime.v1alpha2.PodSandboxConfig.hostname', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log_directory', full_name='runtime.v1alpha2.PodSandboxConfig.log_directory', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dns_config', full_name='runtime.v1alpha2.PodSandboxConfig.dns_config', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port_mappings', full_name='runtime.v1alpha2.PodSandboxConfig.port_mappings', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='runtime.v1alpha2.PodSandboxConfig.labels', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='runtime.v1alpha2.PodSandboxConfig.annotations', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='linux', full_name='runtime.v1alpha2.PodSandboxConfig.linux', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PODSANDBOXCONFIG_LABELSENTRY, _PODSANDBOXCONFIG_ANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1428,
  serialized_end=1939,
)


_RUNPODSANDBOXREQUEST = _descriptor.Descriptor(
  name='RunPodSandboxRequest',
  full_name='runtime.v1alpha2.RunPodSandboxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='config', full_name='runtime.v1alpha2.RunPodSandboxRequest.config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runtime_handler', full_name='runtime.v1alpha2.RunPodSandboxRequest.runtime_handler', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1941,
  serialized_end=2040,
)


_RUNPODSANDBOXRESPONSE = _descriptor.Descriptor(
  name='RunPodSandboxResponse',
  full_name='runtime.v1alpha2.RunPodSandboxResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.RunPodSandboxResponse.pod_sandbox_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2042,
  serialized_end=2089,
)


_STOPPODSANDBOXREQUEST = _descriptor.Descriptor(
  name='StopPodSandboxRequest',
  full_name='runtime.v1alpha2.StopPodSandboxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.StopPodSandboxRequest.pod_sandbox_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2091,
  serialized_end=2138,
)


_STOPPODSANDBOXRESPONSE = _descriptor.Descriptor(
  name='StopPodSandboxResponse',
  full_name='runtime.v1alpha2.StopPodSandboxResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2140,
  serialized_end=2164,
)


_REMOVEPODSANDBOXREQUEST = _descriptor.Descriptor(
  name='RemovePodSandboxRequest',
  full_name='runtime.v1alpha2.RemovePodSandboxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.RemovePodSandboxRequest.pod_sandbox_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2166,
  serialized_end=2215,
)


_REMOVEPODSANDBOXRESPONSE = _descriptor.Descriptor(
  name='RemovePodSandboxResponse',
  full_name='runtime.v1alpha2.RemovePodSandboxResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2217,
  serialized_end=2243,
)


_PODSANDBOXSTATUSREQUEST = _descriptor.Descriptor(
  name='PodSandboxStatusRequest',
  full_name='runtime.v1alpha2.PodSandboxStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.PodSandboxStatusRequest.pod_sandbox_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='verbose', full_name='runtime.v1alpha2.PodSandboxStatusRequest.verbose', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2245,
  serialized_end=2311,
)


_PODIP = _descriptor.Descriptor(
  name='PodIP',
  full_name='runtime.v1alpha2.PodIP',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ip', full_name='runtime.v1alpha2.PodIP.ip', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2313,
  serialized_end=2332,
)


_PODSANDBOXNETWORKSTATUS = _descriptor.Descriptor(
  name='PodSandboxNetworkStatus',
  full_name='runtime.v1alpha2.PodSandboxNetworkStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='ip', full_name='runtime.v1alpha2.PodSandboxNetworkStatus.ip', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='additional_ips', full_name='runtime.v1alpha2.PodSandboxNetworkStatus.additional_ips', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2334,
  serialized_end=2420,
)


_NAMESPACE = _descriptor.Descriptor(
  name='Namespace',
  full_name='runtime.v1alpha2.Namespace',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='options', full_name='runtime.v1alpha2.Namespace.options', index=0,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2422,
  serialized_end=2485,
)


_LINUXPODSANDBOXSTATUS = _descriptor.Descriptor(
  name='LinuxPodSandboxStatus',
  full_name='runtime.v1alpha2.LinuxPodSandboxStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespaces', full_name='runtime.v1alpha2.LinuxPodSandboxStatus.namespaces', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2487,
  serialized_end=2559,
)


_PODSANDBOXSTATUS_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='runtime.v1alpha2.PodSandboxStatus.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandboxStatus.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandboxStatus.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1842,
  serialized_end=1887,
)

_PODSANDBOXSTATUS_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='runtime.v1alpha2.PodSandboxStatus.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandboxStatus.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandboxStatus.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1889,
  serialized_end=1939,
)

_PODSANDBOXSTATUS = _descriptor.Descriptor(
  name='PodSandboxStatus',
  full_name='runtime.v1alpha2.PodSandboxStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.PodSandboxStatus.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='runtime.v1alpha2.PodSandboxStatus.metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.PodSandboxStatus.state', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='runtime.v1alpha2.PodSandboxStatus.created_at', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='network', full_name='runtime.v1alpha2.PodSandboxStatus.network', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='linux', full_name='runtime.v1alpha2.PodSandboxStatus.linux', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='runtime.v1alpha2.PodSandboxStatus.labels', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='runtime.v1alpha2.PodSandboxStatus.annotations', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runtime_handler', full_name='runtime.v1alpha2.PodSandboxStatus.runtime_handler', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PODSANDBOXSTATUS_LABELSENTRY, _PODSANDBOXSTATUS_ANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2562,
  serialized_end=3096,
)


_PODSANDBOXSTATUSRESPONSE_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='runtime.v1alpha2.PodSandboxStatusResponse.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandboxStatusResponse.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandboxStatusResponse.InfoEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3247,
  serialized_end=3290,
)

_PODSANDBOXSTATUSRESPONSE = _descriptor.Descriptor(
  name='PodSandboxStatusResponse',
  full_name='runtime.v1alpha2.PodSandboxStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='runtime.v1alpha2.PodSandboxStatusResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='runtime.v1alpha2.PodSandboxStatusResponse.info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PODSANDBOXSTATUSRESPONSE_INFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3099,
  serialized_end=3290,
)


_PODSANDBOXSTATEVALUE = _descriptor.Descriptor(
  name='PodSandboxStateValue',
  full_name='runtime.v1alpha2.PodSandboxStateValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.PodSandboxStateValue.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3292,
  serialized_end=3364,
)


_PODSANDBOXFILTER_LABELSELECTORENTRY = _descriptor.Descriptor(
  name='LabelSelectorEntry',
  full_name='runtime.v1alpha2.PodSandboxFilter.LabelSelectorEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandboxFilter.LabelSelectorEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandboxFilter.LabelSelectorEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3533,
  serialized_end=3585,
)

_PODSANDBOXFILTER = _descriptor.Descriptor(
  name='PodSandboxFilter',
  full_name='runtime.v1alpha2.PodSandboxFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.PodSandboxFilter.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.PodSandboxFilter.state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_selector', full_name='runtime.v1alpha2.PodSandboxFilter.label_selector', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PODSANDBOXFILTER_LABELSELECTORENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3367,
  serialized_end=3585,
)


_LISTPODSANDBOXREQUEST = _descriptor.Descriptor(
  name='ListPodSandboxRequest',
  full_name='runtime.v1alpha2.ListPodSandboxRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='runtime.v1alpha2.ListPodSandboxRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3587,
  serialized_end=3662,
)


_PODSANDBOX_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='runtime.v1alpha2.PodSandbox.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandbox.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandbox.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1842,
  serialized_end=1887,
)

_PODSANDBOX_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='runtime.v1alpha2.PodSandbox.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.PodSandbox.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.PodSandbox.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1889,
  serialized_end=1939,
)

_PODSANDBOX = _descriptor.Descriptor(
  name='PodSandbox',
  full_name='runtime.v1alpha2.PodSandbox',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.PodSandbox.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='runtime.v1alpha2.PodSandbox.metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.PodSandbox.state', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='runtime.v1alpha2.PodSandbox.created_at', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='runtime.v1alpha2.PodSandbox.labels', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='runtime.v1alpha2.PodSandbox.annotations', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='runtime_handler', full_name='runtime.v1alpha2.PodSandbox.runtime_handler', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PODSANDBOX_LABELSENTRY, _PODSANDBOX_ANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3665,
  serialized_end=4065,
)


_LISTPODSANDBOXRESPONSE = _descriptor.Descriptor(
  name='ListPodSandboxResponse',
  full_name='runtime.v1alpha2.ListPodSandboxResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='items', full_name='runtime.v1alpha2.ListPodSandboxResponse.items', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4067,
  serialized_end=4136,
)


_IMAGESPEC = _descriptor.Descriptor(
  name='ImageSpec',
  full_name='runtime.v1alpha2.ImageSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.ImageSpec.image', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4138,
  serialized_end=4164,
)


_KEYVALUE = _descriptor.Descriptor(
  name='KeyValue',
  full_name='runtime.v1alpha2.KeyValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.KeyValue.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.KeyValue.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4166,
  serialized_end=4204,
)


_LINUXCONTAINERRESOURCES = _descriptor.Descriptor(
  name='LinuxContainerResources',
  full_name='runtime.v1alpha2.LinuxContainerResources',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cpu_period', full_name='runtime.v1alpha2.LinuxContainerResources.cpu_period', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpu_quota', full_name='runtime.v1alpha2.LinuxContainerResources.cpu_quota', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpu_shares', full_name='runtime.v1alpha2.LinuxContainerResources.cpu_shares', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memory_limit_in_bytes', full_name='runtime.v1alpha2.LinuxContainerResources.memory_limit_in_bytes', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='oom_score_adj', full_name='runtime.v1alpha2.LinuxContainerResources.oom_score_adj', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpuset_cpus', full_name='runtime.v1alpha2.LinuxContainerResources.cpuset_cpus', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpuset_mems', full_name='runtime.v1alpha2.LinuxContainerResources.cpuset_mems', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hugepage_limits', full_name='runtime.v1alpha2.LinuxContainerResources.hugepage_limits', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4207,
  serialized_end=4445,
)


_HUGEPAGELIMIT = _descriptor.Descriptor(
  name='HugepageLimit',
  full_name='runtime.v1alpha2.HugepageLimit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='page_size', full_name='runtime.v1alpha2.HugepageLimit.page_size', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='runtime.v1alpha2.HugepageLimit.limit', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4447,
  serialized_end=4496,
)


_SELINUXOPTION = _descriptor.Descriptor(
  name='SELinuxOption',
  full_name='runtime.v1alpha2.SELinuxOption',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user', full_name='runtime.v1alpha2.SELinuxOption.user', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='role', full_name='runtime.v1alpha2.SELinuxOption.role', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='type', full_name='runtime.v1alpha2.SELinuxOption.type', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='level', full_name='runtime.v1alpha2.SELinuxOption.level', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4498,
  serialized_end=4570,
)


_CAPABILITY = _descriptor.Descriptor(
  name='Capability',
  full_name='runtime.v1alpha2.Capability',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='add_capabilities', full_name='runtime.v1alpha2.Capability.add_capabilities', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='drop_capabilities', full_name='runtime.v1alpha2.Capability.drop_capabilities', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4572,
  serialized_end=4637,
)


_LINUXCONTAINERSECURITYCONTEXT = _descriptor.Descriptor(
  name='LinuxContainerSecurityContext',
  full_name='runtime.v1alpha2.LinuxContainerSecurityContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='capabilities', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.capabilities', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='privileged', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.privileged', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='namespace_options', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.namespace_options', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='selinux_options', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.selinux_options', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_as_user', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.run_as_user', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_as_group', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.run_as_group', index=5,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_as_username', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.run_as_username', index=6,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='readonly_rootfs', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.readonly_rootfs', index=7,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='supplemental_groups', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.supplemental_groups', index=8,
      number=8, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='apparmor_profile', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.apparmor_profile', index=9,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='seccomp_profile_path', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.seccomp_profile_path', index=10,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='no_new_privs', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.no_new_privs', index=11,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='masked_paths', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.masked_paths', index=12,
      number=13, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='readonly_paths', full_name='runtime.v1alpha2.LinuxContainerSecurityContext.readonly_paths', index=13,
      number=14, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4640,
  serialized_end=5169,
)


_LINUXCONTAINERCONFIG = _descriptor.Descriptor(
  name='LinuxContainerConfig',
  full_name='runtime.v1alpha2.LinuxContainerConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='resources', full_name='runtime.v1alpha2.LinuxContainerConfig.resources', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='security_context', full_name='runtime.v1alpha2.LinuxContainerConfig.security_context', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5172,
  serialized_end=5331,
)


_WINDOWSCONTAINERSECURITYCONTEXT = _descriptor.Descriptor(
  name='WindowsContainerSecurityContext',
  full_name='runtime.v1alpha2.WindowsContainerSecurityContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='run_as_username', full_name='runtime.v1alpha2.WindowsContainerSecurityContext.run_as_username', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='credential_spec', full_name='runtime.v1alpha2.WindowsContainerSecurityContext.credential_spec', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5333,
  serialized_end=5416,
)


_WINDOWSCONTAINERCONFIG = _descriptor.Descriptor(
  name='WindowsContainerConfig',
  full_name='runtime.v1alpha2.WindowsContainerConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='resources', full_name='runtime.v1alpha2.WindowsContainerConfig.resources', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='security_context', full_name='runtime.v1alpha2.WindowsContainerConfig.security_context', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5419,
  serialized_end=5584,
)


_WINDOWSCONTAINERRESOURCES = _descriptor.Descriptor(
  name='WindowsContainerResources',
  full_name='runtime.v1alpha2.WindowsContainerResources',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='cpu_shares', full_name='runtime.v1alpha2.WindowsContainerResources.cpu_shares', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpu_count', full_name='runtime.v1alpha2.WindowsContainerResources.cpu_count', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpu_maximum', full_name='runtime.v1alpha2.WindowsContainerResources.cpu_maximum', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memory_limit_in_bytes', full_name='runtime.v1alpha2.WindowsContainerResources.memory_limit_in_bytes', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5586,
  serialized_end=5704,
)


_CONTAINERMETADATA = _descriptor.Descriptor(
  name='ContainerMetadata',
  full_name='runtime.v1alpha2.ContainerMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='runtime.v1alpha2.ContainerMetadata.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='attempt', full_name='runtime.v1alpha2.ContainerMetadata.attempt', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5706,
  serialized_end=5756,
)


_DEVICE = _descriptor.Descriptor(
  name='Device',
  full_name='runtime.v1alpha2.Device',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_path', full_name='runtime.v1alpha2.Device.container_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='host_path', full_name='runtime.v1alpha2.Device.host_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='permissions', full_name='runtime.v1alpha2.Device.permissions', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5758,
  serialized_end=5830,
)


_CONTAINERCONFIG_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='runtime.v1alpha2.ContainerConfig.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerConfig.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerConfig.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1842,
  serialized_end=1887,
)

_CONTAINERCONFIG_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='runtime.v1alpha2.ContainerConfig.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerConfig.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerConfig.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1889,
  serialized_end=1939,
)

_CONTAINERCONFIG = _descriptor.Descriptor(
  name='ContainerConfig',
  full_name='runtime.v1alpha2.ContainerConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='runtime.v1alpha2.ContainerConfig.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.ContainerConfig.image', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='command', full_name='runtime.v1alpha2.ContainerConfig.command', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='args', full_name='runtime.v1alpha2.ContainerConfig.args', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='working_dir', full_name='runtime.v1alpha2.ContainerConfig.working_dir', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='envs', full_name='runtime.v1alpha2.ContainerConfig.envs', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mounts', full_name='runtime.v1alpha2.ContainerConfig.mounts', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='devices', full_name='runtime.v1alpha2.ContainerConfig.devices', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='runtime.v1alpha2.ContainerConfig.labels', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='runtime.v1alpha2.ContainerConfig.annotations', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log_path', full_name='runtime.v1alpha2.ContainerConfig.log_path', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stdin', full_name='runtime.v1alpha2.ContainerConfig.stdin', index=11,
      number=12, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stdin_once', full_name='runtime.v1alpha2.ContainerConfig.stdin_once', index=12,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tty', full_name='runtime.v1alpha2.ContainerConfig.tty', index=13,
      number=14, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='linux', full_name='runtime.v1alpha2.ContainerConfig.linux', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='windows', full_name='runtime.v1alpha2.ContainerConfig.windows', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINERCONFIG_LABELSENTRY, _CONTAINERCONFIG_ANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5833,
  serialized_end=6542,
)


_CREATECONTAINERREQUEST = _descriptor.Descriptor(
  name='CreateContainerRequest',
  full_name='runtime.v1alpha2.CreateContainerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.CreateContainerRequest.pod_sandbox_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='config', full_name='runtime.v1alpha2.CreateContainerRequest.config', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sandbox_config', full_name='runtime.v1alpha2.CreateContainerRequest.sandbox_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6545,
  serialized_end=6704,
)


_CREATECONTAINERRESPONSE = _descriptor.Descriptor(
  name='CreateContainerResponse',
  full_name='runtime.v1alpha2.CreateContainerResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.CreateContainerResponse.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6706,
  serialized_end=6753,
)


_STARTCONTAINERREQUEST = _descriptor.Descriptor(
  name='StartContainerRequest',
  full_name='runtime.v1alpha2.StartContainerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.StartContainerRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6755,
  serialized_end=6800,
)


_STARTCONTAINERRESPONSE = _descriptor.Descriptor(
  name='StartContainerResponse',
  full_name='runtime.v1alpha2.StartContainerResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6802,
  serialized_end=6826,
)


_STOPCONTAINERREQUEST = _descriptor.Descriptor(
  name='StopContainerRequest',
  full_name='runtime.v1alpha2.StopContainerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.StopContainerRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='runtime.v1alpha2.StopContainerRequest.timeout', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6828,
  serialized_end=6889,
)


_STOPCONTAINERRESPONSE = _descriptor.Descriptor(
  name='StopContainerResponse',
  full_name='runtime.v1alpha2.StopContainerResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6891,
  serialized_end=6914,
)


_REMOVECONTAINERREQUEST = _descriptor.Descriptor(
  name='RemoveContainerRequest',
  full_name='runtime.v1alpha2.RemoveContainerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.RemoveContainerRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6916,
  serialized_end=6962,
)


_REMOVECONTAINERRESPONSE = _descriptor.Descriptor(
  name='RemoveContainerResponse',
  full_name='runtime.v1alpha2.RemoveContainerResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6964,
  serialized_end=6989,
)


_CONTAINERSTATEVALUE = _descriptor.Descriptor(
  name='ContainerStateValue',
  full_name='runtime.v1alpha2.ContainerStateValue',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.ContainerStateValue.state', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=6991,
  serialized_end=7061,
)


_CONTAINERFILTER_LABELSELECTORENTRY = _descriptor.Descriptor(
  name='LabelSelectorEntry',
  full_name='runtime.v1alpha2.ContainerFilter.LabelSelectorEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerFilter.LabelSelectorEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerFilter.LabelSelectorEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3533,
  serialized_end=3585,
)

_CONTAINERFILTER = _descriptor.Descriptor(
  name='ContainerFilter',
  full_name='runtime.v1alpha2.ContainerFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.ContainerFilter.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.ContainerFilter.state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.ContainerFilter.pod_sandbox_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_selector', full_name='runtime.v1alpha2.ContainerFilter.label_selector', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINERFILTER_LABELSELECTORENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7064,
  serialized_end=7303,
)


_LISTCONTAINERSREQUEST = _descriptor.Descriptor(
  name='ListContainersRequest',
  full_name='runtime.v1alpha2.ListContainersRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='runtime.v1alpha2.ListContainersRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7305,
  serialized_end=7379,
)


_CONTAINER_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='runtime.v1alpha2.Container.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.Container.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.Container.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1842,
  serialized_end=1887,
)

_CONTAINER_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='runtime.v1alpha2.Container.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.Container.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.Container.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1889,
  serialized_end=1939,
)

_CONTAINER = _descriptor.Descriptor(
  name='Container',
  full_name='runtime.v1alpha2.Container',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.Container.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.Container.pod_sandbox_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='runtime.v1alpha2.Container.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.Container.image', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_ref', full_name='runtime.v1alpha2.Container.image_ref', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.Container.state', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='runtime.v1alpha2.Container.created_at', index=6,
      number=7, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='runtime.v1alpha2.Container.labels', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='runtime.v1alpha2.Container.annotations', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINER_LABELSENTRY, _CONTAINER_ANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7382,
  serialized_end=7839,
)


_LISTCONTAINERSRESPONSE = _descriptor.Descriptor(
  name='ListContainersResponse',
  full_name='runtime.v1alpha2.ListContainersResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='containers', full_name='runtime.v1alpha2.ListContainersResponse.containers', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7841,
  serialized_end=7914,
)


_CONTAINERSTATUSREQUEST = _descriptor.Descriptor(
  name='ContainerStatusRequest',
  full_name='runtime.v1alpha2.ContainerStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.ContainerStatusRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='verbose', full_name='runtime.v1alpha2.ContainerStatusRequest.verbose', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7916,
  serialized_end=7979,
)


_CONTAINERSTATUS_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='runtime.v1alpha2.ContainerStatus.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerStatus.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerStatus.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1842,
  serialized_end=1887,
)

_CONTAINERSTATUS_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='runtime.v1alpha2.ContainerStatus.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerStatus.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerStatus.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1889,
  serialized_end=1939,
)

_CONTAINERSTATUS = _descriptor.Descriptor(
  name='ContainerStatus',
  full_name='runtime.v1alpha2.ContainerStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.ContainerStatus.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='runtime.v1alpha2.ContainerStatus.metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='runtime.v1alpha2.ContainerStatus.state', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='created_at', full_name='runtime.v1alpha2.ContainerStatus.created_at', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='started_at', full_name='runtime.v1alpha2.ContainerStatus.started_at', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='finished_at', full_name='runtime.v1alpha2.ContainerStatus.finished_at', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exit_code', full_name='runtime.v1alpha2.ContainerStatus.exit_code', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.ContainerStatus.image', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='image_ref', full_name='runtime.v1alpha2.ContainerStatus.image_ref', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reason', full_name='runtime.v1alpha2.ContainerStatus.reason', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='runtime.v1alpha2.ContainerStatus.message', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='runtime.v1alpha2.ContainerStatus.labels', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='runtime.v1alpha2.ContainerStatus.annotations', index=12,
      number=13, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mounts', full_name='runtime.v1alpha2.ContainerStatus.mounts', index=13,
      number=14, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='log_path', full_name='runtime.v1alpha2.ContainerStatus.log_path', index=14,
      number=15, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINERSTATUS_LABELSENTRY, _CONTAINERSTATUS_ANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=7982,
  serialized_end=8585,
)


_CONTAINERSTATUSRESPONSE_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='runtime.v1alpha2.ContainerStatusResponse.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerStatusResponse.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerStatusResponse.InfoEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3247,
  serialized_end=3290,
)

_CONTAINERSTATUSRESPONSE = _descriptor.Descriptor(
  name='ContainerStatusResponse',
  full_name='runtime.v1alpha2.ContainerStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='runtime.v1alpha2.ContainerStatusResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='runtime.v1alpha2.ContainerStatusResponse.info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINERSTATUSRESPONSE_INFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8588,
  serialized_end=8776,
)


_UPDATECONTAINERRESOURCESREQUEST = _descriptor.Descriptor(
  name='UpdateContainerResourcesRequest',
  full_name='runtime.v1alpha2.UpdateContainerResourcesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.UpdateContainerResourcesRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='linux', full_name='runtime.v1alpha2.UpdateContainerResourcesRequest.linux', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8778,
  serialized_end=8891,
)


_UPDATECONTAINERRESOURCESRESPONSE = _descriptor.Descriptor(
  name='UpdateContainerResourcesResponse',
  full_name='runtime.v1alpha2.UpdateContainerResourcesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8893,
  serialized_end=8927,
)


_EXECSYNCREQUEST = _descriptor.Descriptor(
  name='ExecSyncRequest',
  full_name='runtime.v1alpha2.ExecSyncRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.ExecSyncRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cmd', full_name='runtime.v1alpha2.ExecSyncRequest.cmd', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='runtime.v1alpha2.ExecSyncRequest.timeout', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=8929,
  serialized_end=8998,
)


_EXECSYNCRESPONSE = _descriptor.Descriptor(
  name='ExecSyncResponse',
  full_name='runtime.v1alpha2.ExecSyncResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stdout', full_name='runtime.v1alpha2.ExecSyncResponse.stdout', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stderr', full_name='runtime.v1alpha2.ExecSyncResponse.stderr', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='exit_code', full_name='runtime.v1alpha2.ExecSyncResponse.exit_code', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9000,
  serialized_end=9069,
)


_EXECREQUEST = _descriptor.Descriptor(
  name='ExecRequest',
  full_name='runtime.v1alpha2.ExecRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.ExecRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cmd', full_name='runtime.v1alpha2.ExecRequest.cmd', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tty', full_name='runtime.v1alpha2.ExecRequest.tty', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stdin', full_name='runtime.v1alpha2.ExecRequest.stdin', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stdout', full_name='runtime.v1alpha2.ExecRequest.stdout', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stderr', full_name='runtime.v1alpha2.ExecRequest.stderr', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9071,
  serialized_end=9179,
)


_EXECRESPONSE = _descriptor.Descriptor(
  name='ExecResponse',
  full_name='runtime.v1alpha2.ExecResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='runtime.v1alpha2.ExecResponse.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9181,
  serialized_end=9208,
)


_ATTACHREQUEST = _descriptor.Descriptor(
  name='AttachRequest',
  full_name='runtime.v1alpha2.AttachRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.AttachRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stdin', full_name='runtime.v1alpha2.AttachRequest.stdin', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tty', full_name='runtime.v1alpha2.AttachRequest.tty', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stdout', full_name='runtime.v1alpha2.AttachRequest.stdout', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stderr', full_name='runtime.v1alpha2.AttachRequest.stderr', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9210,
  serialized_end=9307,
)


_ATTACHRESPONSE = _descriptor.Descriptor(
  name='AttachResponse',
  full_name='runtime.v1alpha2.AttachResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='runtime.v1alpha2.AttachResponse.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9309,
  serialized_end=9338,
)


_PORTFORWARDREQUEST = _descriptor.Descriptor(
  name='PortForwardRequest',
  full_name='runtime.v1alpha2.PortForwardRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.PortForwardRequest.pod_sandbox_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='runtime.v1alpha2.PortForwardRequest.port', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9340,
  serialized_end=9398,
)


_PORTFORWARDRESPONSE = _descriptor.Descriptor(
  name='PortForwardResponse',
  full_name='runtime.v1alpha2.PortForwardResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='url', full_name='runtime.v1alpha2.PortForwardResponse.url', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9400,
  serialized_end=9434,
)


_IMAGEFILTER = _descriptor.Descriptor(
  name='ImageFilter',
  full_name='runtime.v1alpha2.ImageFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.ImageFilter.image', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9436,
  serialized_end=9493,
)


_LISTIMAGESREQUEST = _descriptor.Descriptor(
  name='ListImagesRequest',
  full_name='runtime.v1alpha2.ListImagesRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='runtime.v1alpha2.ListImagesRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9495,
  serialized_end=9561,
)


_IMAGE = _descriptor.Descriptor(
  name='Image',
  full_name='runtime.v1alpha2.Image',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.Image.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repo_tags', full_name='runtime.v1alpha2.Image.repo_tags', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='repo_digests', full_name='runtime.v1alpha2.Image.repo_digests', index=2,
      number=3, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='size', full_name='runtime.v1alpha2.Image.size', index=3,
      number=4, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uid', full_name='runtime.v1alpha2.Image.uid', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='username', full_name='runtime.v1alpha2.Image.username', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9564,
  serialized_end=9699,
)


_LISTIMAGESRESPONSE = _descriptor.Descriptor(
  name='ListImagesResponse',
  full_name='runtime.v1alpha2.ListImagesResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='images', full_name='runtime.v1alpha2.ListImagesResponse.images', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9701,
  serialized_end=9762,
)


_IMAGESTATUSREQUEST = _descriptor.Descriptor(
  name='ImageStatusRequest',
  full_name='runtime.v1alpha2.ImageStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.ImageStatusRequest.image', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='verbose', full_name='runtime.v1alpha2.ImageStatusRequest.verbose', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9764,
  serialized_end=9845,
)


_IMAGESTATUSRESPONSE_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='runtime.v1alpha2.ImageStatusResponse.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ImageStatusResponse.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ImageStatusResponse.InfoEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3247,
  serialized_end=3290,
)

_IMAGESTATUSRESPONSE = _descriptor.Descriptor(
  name='ImageStatusResponse',
  full_name='runtime.v1alpha2.ImageStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.ImageStatusResponse.image', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='runtime.v1alpha2.ImageStatusResponse.info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_IMAGESTATUSRESPONSE_INFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=9848,
  serialized_end=10017,
)


_AUTHCONFIG = _descriptor.Descriptor(
  name='AuthConfig',
  full_name='runtime.v1alpha2.AuthConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='username', full_name='runtime.v1alpha2.AuthConfig.username', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='password', full_name='runtime.v1alpha2.AuthConfig.password', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='auth', full_name='runtime.v1alpha2.AuthConfig.auth', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='server_address', full_name='runtime.v1alpha2.AuthConfig.server_address', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='identity_token', full_name='runtime.v1alpha2.AuthConfig.identity_token', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='registry_token', full_name='runtime.v1alpha2.AuthConfig.registry_token', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10020,
  serialized_end=10154,
)


_PULLIMAGEREQUEST = _descriptor.Descriptor(
  name='PullImageRequest',
  full_name='runtime.v1alpha2.PullImageRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.PullImageRequest.image', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='auth', full_name='runtime.v1alpha2.PullImageRequest.auth', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sandbox_config', full_name='runtime.v1alpha2.PullImageRequest.sandbox_config', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10157,
  serialized_end=10323,
)


_PULLIMAGERESPONSE = _descriptor.Descriptor(
  name='PullImageResponse',
  full_name='runtime.v1alpha2.PullImageResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image_ref', full_name='runtime.v1alpha2.PullImageResponse.image_ref', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10325,
  serialized_end=10363,
)


_REMOVEIMAGEREQUEST = _descriptor.Descriptor(
  name='RemoveImageRequest',
  full_name='runtime.v1alpha2.RemoveImageRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image', full_name='runtime.v1alpha2.RemoveImageRequest.image', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10365,
  serialized_end=10429,
)


_REMOVEIMAGERESPONSE = _descriptor.Descriptor(
  name='RemoveImageResponse',
  full_name='runtime.v1alpha2.RemoveImageResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10431,
  serialized_end=10452,
)


_NETWORKCONFIG = _descriptor.Descriptor(
  name='NetworkConfig',
  full_name='runtime.v1alpha2.NetworkConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='pod_cidr', full_name='runtime.v1alpha2.NetworkConfig.pod_cidr', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10454,
  serialized_end=10487,
)


_RUNTIMECONFIG = _descriptor.Descriptor(
  name='RuntimeConfig',
  full_name='runtime.v1alpha2.RuntimeConfig',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='network_config', full_name='runtime.v1alpha2.RuntimeConfig.network_config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10489,
  serialized_end=10561,
)


_UPDATERUNTIMECONFIGREQUEST = _descriptor.Descriptor(
  name='UpdateRuntimeConfigRequest',
  full_name='runtime.v1alpha2.UpdateRuntimeConfigRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='runtime_config', full_name='runtime.v1alpha2.UpdateRuntimeConfigRequest.runtime_config', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10563,
  serialized_end=10648,
)


_UPDATERUNTIMECONFIGRESPONSE = _descriptor.Descriptor(
  name='UpdateRuntimeConfigResponse',
  full_name='runtime.v1alpha2.UpdateRuntimeConfigResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10650,
  serialized_end=10679,
)


_RUNTIMECONDITION = _descriptor.Descriptor(
  name='RuntimeCondition',
  full_name='runtime.v1alpha2.RuntimeCondition',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='type', full_name='runtime.v1alpha2.RuntimeCondition.type', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='status', full_name='runtime.v1alpha2.RuntimeCondition.status', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='reason', full_name='runtime.v1alpha2.RuntimeCondition.reason', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message', full_name='runtime.v1alpha2.RuntimeCondition.message', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10681,
  serialized_end=10762,
)


_RUNTIMESTATUS = _descriptor.Descriptor(
  name='RuntimeStatus',
  full_name='runtime.v1alpha2.RuntimeStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='conditions', full_name='runtime.v1alpha2.RuntimeStatus.conditions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10764,
  serialized_end=10835,
)


_STATUSREQUEST = _descriptor.Descriptor(
  name='StatusRequest',
  full_name='runtime.v1alpha2.StatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='verbose', full_name='runtime.v1alpha2.StatusRequest.verbose', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10837,
  serialized_end=10869,
)


_STATUSRESPONSE_INFOENTRY = _descriptor.Descriptor(
  name='InfoEntry',
  full_name='runtime.v1alpha2.StatusResponse.InfoEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.StatusResponse.InfoEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.StatusResponse.InfoEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3247,
  serialized_end=3290,
)

_STATUSRESPONSE = _descriptor.Descriptor(
  name='StatusResponse',
  full_name='runtime.v1alpha2.StatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='runtime.v1alpha2.StatusResponse.status', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='info', full_name='runtime.v1alpha2.StatusResponse.info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_STATUSRESPONSE_INFOENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=10872,
  serialized_end=11040,
)


_IMAGEFSINFOREQUEST = _descriptor.Descriptor(
  name='ImageFsInfoRequest',
  full_name='runtime.v1alpha2.ImageFsInfoRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11042,
  serialized_end=11062,
)


_UINT64VALUE = _descriptor.Descriptor(
  name='UInt64Value',
  full_name='runtime.v1alpha2.UInt64Value',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.UInt64Value.value', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11064,
  serialized_end=11092,
)


_FILESYSTEMIDENTIFIER = _descriptor.Descriptor(
  name='FilesystemIdentifier',
  full_name='runtime.v1alpha2.FilesystemIdentifier',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mountpoint', full_name='runtime.v1alpha2.FilesystemIdentifier.mountpoint', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11094,
  serialized_end=11136,
)


_FILESYSTEMUSAGE = _descriptor.Descriptor(
  name='FilesystemUsage',
  full_name='runtime.v1alpha2.FilesystemUsage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='runtime.v1alpha2.FilesystemUsage.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fs_id', full_name='runtime.v1alpha2.FilesystemUsage.fs_id', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='used_bytes', full_name='runtime.v1alpha2.FilesystemUsage.used_bytes', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='inodes_used', full_name='runtime.v1alpha2.FilesystemUsage.inodes_used', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11139,
  serialized_end=11333,
)


_IMAGEFSINFORESPONSE = _descriptor.Descriptor(
  name='ImageFsInfoResponse',
  full_name='runtime.v1alpha2.ImageFsInfoResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='image_filesystems', full_name='runtime.v1alpha2.ImageFsInfoResponse.image_filesystems', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11335,
  serialized_end=11418,
)


_CONTAINERSTATSREQUEST = _descriptor.Descriptor(
  name='ContainerStatsRequest',
  full_name='runtime.v1alpha2.ContainerStatsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.ContainerStatsRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11420,
  serialized_end=11465,
)


_CONTAINERSTATSRESPONSE = _descriptor.Descriptor(
  name='ContainerStatsResponse',
  full_name='runtime.v1alpha2.ContainerStatsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stats', full_name='runtime.v1alpha2.ContainerStatsResponse.stats', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11467,
  serialized_end=11540,
)


_LISTCONTAINERSTATSREQUEST = _descriptor.Descriptor(
  name='ListContainerStatsRequest',
  full_name='runtime.v1alpha2.ListContainerStatsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='runtime.v1alpha2.ListContainerStatsRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11542,
  serialized_end=11625,
)


_CONTAINERSTATSFILTER_LABELSELECTORENTRY = _descriptor.Descriptor(
  name='LabelSelectorEntry',
  full_name='runtime.v1alpha2.ContainerStatsFilter.LabelSelectorEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerStatsFilter.LabelSelectorEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerStatsFilter.LabelSelectorEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3533,
  serialized_end=3585,
)

_CONTAINERSTATSFILTER = _descriptor.Descriptor(
  name='ContainerStatsFilter',
  full_name='runtime.v1alpha2.ContainerStatsFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.ContainerStatsFilter.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pod_sandbox_id', full_name='runtime.v1alpha2.ContainerStatsFilter.pod_sandbox_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='label_selector', full_name='runtime.v1alpha2.ContainerStatsFilter.label_selector', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINERSTATSFILTER_LABELSELECTORENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11628,
  serialized_end=11823,
)


_LISTCONTAINERSTATSRESPONSE = _descriptor.Descriptor(
  name='ListContainerStatsResponse',
  full_name='runtime.v1alpha2.ListContainerStatsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='stats', full_name='runtime.v1alpha2.ListContainerStatsResponse.stats', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11825,
  serialized_end=11902,
)


_CONTAINERATTRIBUTES_LABELSENTRY = _descriptor.Descriptor(
  name='LabelsEntry',
  full_name='runtime.v1alpha2.ContainerAttributes.LabelsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerAttributes.LabelsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerAttributes.LabelsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1842,
  serialized_end=1887,
)

_CONTAINERATTRIBUTES_ANNOTATIONSENTRY = _descriptor.Descriptor(
  name='AnnotationsEntry',
  full_name='runtime.v1alpha2.ContainerAttributes.AnnotationsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='runtime.v1alpha2.ContainerAttributes.AnnotationsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='runtime.v1alpha2.ContainerAttributes.AnnotationsEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1889,
  serialized_end=1939,
)

_CONTAINERATTRIBUTES = _descriptor.Descriptor(
  name='ContainerAttributes',
  full_name='runtime.v1alpha2.ContainerAttributes',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='runtime.v1alpha2.ContainerAttributes.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='runtime.v1alpha2.ContainerAttributes.metadata', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='labels', full_name='runtime.v1alpha2.ContainerAttributes.labels', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='annotations', full_name='runtime.v1alpha2.ContainerAttributes.annotations', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_CONTAINERATTRIBUTES_LABELSENTRY, _CONTAINERATTRIBUTES_ANNOTATIONSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=11905,
  serialized_end=12236,
)


_CONTAINERSTATS = _descriptor.Descriptor(
  name='ContainerStats',
  full_name='runtime.v1alpha2.ContainerStats',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='attributes', full_name='runtime.v1alpha2.ContainerStats.attributes', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cpu', full_name='runtime.v1alpha2.ContainerStats.cpu', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='memory', full_name='runtime.v1alpha2.ContainerStats.memory', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='writable_layer', full_name='runtime.v1alpha2.ContainerStats.writable_layer', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12239,
  serialized_end=12461,
)


_CPUUSAGE = _descriptor.Descriptor(
  name='CpuUsage',
  full_name='runtime.v1alpha2.CpuUsage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='runtime.v1alpha2.CpuUsage.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='usage_core_nano_seconds', full_name='runtime.v1alpha2.CpuUsage.usage_core_nano_seconds', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12463,
  serialized_end=12556,
)


_MEMORYUSAGE = _descriptor.Descriptor(
  name='MemoryUsage',
  full_name='runtime.v1alpha2.MemoryUsage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='runtime.v1alpha2.MemoryUsage.timestamp', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='working_set_bytes', full_name='runtime.v1alpha2.MemoryUsage.working_set_bytes', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12558,
  serialized_end=12648,
)


_REOPENCONTAINERLOGREQUEST = _descriptor.Descriptor(
  name='ReopenContainerLogRequest',
  full_name='runtime.v1alpha2.ReopenContainerLogRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='container_id', full_name='runtime.v1alpha2.ReopenContainerLogRequest.container_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12650,
  serialized_end=12699,
)


_REOPENCONTAINERLOGRESPONSE = _descriptor.Descriptor(
  name='ReopenContainerLogResponse',
  full_name='runtime.v1alpha2.ReopenContainerLogResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=12701,
  serialized_end=12729,
)

_PORTMAPPING.fields_by_name['protocol'].enum_type = _PROTOCOL
_MOUNT.fields_by_name['propagation'].enum_type = _MOUNTPROPAGATION
_NAMESPACEOPTION.fields_by_name['network'].enum_type = _NAMESPACEMODE
_NAMESPACEOPTION.fields_by_name['pid'].enum_type = _NAMESPACEMODE
_NAMESPACEOPTION.fields_by_name['ipc'].enum_type = _NAMESPACEMODE
_LINUXSANDBOXSECURITYCONTEXT.fields_by_name['namespace_options'].message_type = _NAMESPACEOPTION
_LINUXSANDBOXSECURITYCONTEXT.fields_by_name['selinux_options'].message_type = _SELINUXOPTION
_LINUXSANDBOXSECURITYCONTEXT.fields_by_name['run_as_user'].message_type = _INT64VALUE
_LINUXSANDBOXSECURITYCONTEXT.fields_by_name['run_as_group'].message_type = _INT64VALUE
_LINUXPODSANDBOXCONFIG_SYSCTLSENTRY.containing_type = _LINUXPODSANDBOXCONFIG
_LINUXPODSANDBOXCONFIG.fields_by_name['security_context'].message_type = _LINUXSANDBOXSECURITYCONTEXT
_LINUXPODSANDBOXCONFIG.fields_by_name['sysctls'].message_type = _LINUXPODSANDBOXCONFIG_SYSCTLSENTRY
_PODSANDBOXCONFIG_LABELSENTRY.containing_type = _PODSANDBOXCONFIG
_PODSANDBOXCONFIG_ANNOTATIONSENTRY.containing_type = _PODSANDBOXCONFIG
_PODSANDBOXCONFIG.fields_by_name['metadata'].message_type = _PODSANDBOXMETADATA
_PODSANDBOXCONFIG.fields_by_name['dns_config'].message_type = _DNSCONFIG
_PODSANDBOXCONFIG.fields_by_name['port_mappings'].message_type = _PORTMAPPING
_PODSANDBOXCONFIG.fields_by_name['labels'].message_type = _PODSANDBOXCONFIG_LABELSENTRY
_PODSANDBOXCONFIG.fields_by_name['annotations'].message_type = _PODSANDBOXCONFIG_ANNOTATIONSENTRY
_PODSANDBOXCONFIG.fields_by_name['linux'].message_type = _LINUXPODSANDBOXCONFIG
_RUNPODSANDBOXREQUEST.fields_by_name['config'].message_type = _PODSANDBOXCONFIG
_PODSANDBOXNETWORKSTATUS.fields_by_name['additional_ips'].message_type = _PODIP
_NAMESPACE.fields_by_name['options'].message_type = _NAMESPACEOPTION
_LINUXPODSANDBOXSTATUS.fields_by_name['namespaces'].message_type = _NAMESPACE
_PODSANDBOXSTATUS_LABELSENTRY.containing_type = _PODSANDBOXSTATUS
_PODSANDBOXSTATUS_ANNOTATIONSENTRY.containing_type = _PODSANDBOXSTATUS
_PODSANDBOXSTATUS.fields_by_name['metadata'].message_type = _PODSANDBOXMETADATA
_PODSANDBOXSTATUS.fields_by_name['state'].enum_type = _PODSANDBOXSTATE
_PODSANDBOXSTATUS.fields_by_name['network'].message_type = _PODSANDBOXNETWORKSTATUS
_PODSANDBOXSTATUS.fields_by_name['linux'].message_type = _LINUXPODSANDBOXSTATUS
_PODSANDBOXSTATUS.fields_by_name['labels'].message_type = _PODSANDBOXSTATUS_LABELSENTRY
_PODSANDBOXSTATUS.fields_by_name['annotations'].message_type = _PODSANDBOXSTATUS_ANNOTATIONSENTRY
_PODSANDBOXSTATUSRESPONSE_INFOENTRY.containing_type = _PODSANDBOXSTATUSRESPONSE
_PODSANDBOXSTATUSRESPONSE.fields_by_name['status'].message_type = _PODSANDBOXSTATUS
_PODSANDBOXSTATUSRESPONSE.fields_by_name['info'].message_type = _PODSANDBOXSTATUSRESPONSE_INFOENTRY
_PODSANDBOXSTATEVALUE.fields_by_name['state'].enum_type = _PODSANDBOXSTATE
_PODSANDBOXFILTER_LABELSELECTORENTRY.containing_type = _PODSANDBOXFILTER
_PODSANDBOXFILTER.fields_by_name['state'].message_type = _PODSANDBOXSTATEVALUE
_PODSANDBOXFILTER.fields_by_name['label_selector'].message_type = _PODSANDBOXFILTER_LABELSELECTORENTRY
_LISTPODSANDBOXREQUEST.fields_by_name['filter'].message_type = _PODSANDBOXFILTER
_PODSANDBOX_LABELSENTRY.containing_type = _PODSANDBOX
_PODSANDBOX_ANNOTATIONSENTRY.containing_type = _PODSANDBOX
_PODSANDBOX.fields_by_name['metadata'].message_type = _PODSANDBOXMETADATA
_PODSANDBOX.fields_by_name['state'].enum_type = _PODSANDBOXSTATE
_PODSANDBOX.fields_by_name['labels'].message_type = _PODSANDBOX_LABELSENTRY
_PODSANDBOX.fields_by_name['annotations'].message_type = _PODSANDBOX_ANNOTATIONSENTRY
_LISTPODSANDBOXRESPONSE.fields_by_name['items'].message_type = _PODSANDBOX
_LINUXCONTAINERRESOURCES.fields_by_name['hugepage_limits'].message_type = _HUGEPAGELIMIT
_LINUXCONTAINERSECURITYCONTEXT.fields_by_name['capabilities'].message_type = _CAPABILITY
_LINUXCONTAINERSECURITYCONTEXT.fields_by_name['namespace_options'].message_type = _NAMESPACEOPTION
_LINUXCONTAINERSECURITYCONTEXT.fields_by_name['selinux_options'].message_type = _SELINUXOPTION
_LINUXCONTAINERSECURITYCONTEXT.fields_by_name['run_as_user'].message_type = _INT64VALUE
_LINUXCONTAINERSECURITYCONTEXT.fields_by_name['run_as_group'].message_type = _INT64VALUE
_LINUXCONTAINERCONFIG.fields_by_name['resources'].message_type = _LINUXCONTAINERRESOURCES
_LINUXCONTAINERCONFIG.fields_by_name['security_context'].message_type = _LINUXCONTAINERSECURITYCONTEXT
_WINDOWSCONTAINERCONFIG.fields_by_name['resources'].message_type = _WINDOWSCONTAINERRESOURCES
_WINDOWSCONTAINERCONFIG.fields_by_name['security_context'].message_type = _WINDOWSCONTAINERSECURITYCONTEXT
_CONTAINERCONFIG_LABELSENTRY.containing_type = _CONTAINERCONFIG
_CONTAINERCONFIG_ANNOTATIONSENTRY.containing_type = _CONTAINERCONFIG
_CONTAINERCONFIG.fields_by_name['metadata'].message_type = _CONTAINERMETADATA
_CONTAINERCONFIG.fields_by_name['image'].message_type = _IMAGESPEC
_CONTAINERCONFIG.fields_by_name['envs'].message_type = _KEYVALUE
_CONTAINERCONFIG.fields_by_name['mounts'].message_type = _MOUNT
_CONTAINERCONFIG.fields_by_name['devices'].message_type = _DEVICE
_CONTAINERCONFIG.fields_by_name['labels'].message_type = _CONTAINERCONFIG_LABELSENTRY
_CONTAINERCONFIG.fields_by_name['annotations'].message_type = _CONTAINERCONFIG_ANNOTATIONSENTRY
_CONTAINERCONFIG.fields_by_name['linux'].message_type = _LINUXCONTAINERCONFIG
_CONTAINERCONFIG.fields_by_name['windows'].message_type = _WINDOWSCONTAINERCONFIG
_CREATECONTAINERREQUEST.fields_by_name['config'].message_type = _CONTAINERCONFIG
_CREATECONTAINERREQUEST.fields_by_name['sandbox_config'].message_type = _PODSANDBOXCONFIG
_CONTAINERSTATEVALUE.fields_by_name['state'].enum_type = _CONTAINERSTATE
_CONTAINERFILTER_LABELSELECTORENTRY.containing_type = _CONTAINERFILTER
_CONTAINERFILTER.fields_by_name['state'].message_type = _CONTAINERSTATEVALUE
_CONTAINERFILTER.fields_by_name['label_selector'].message_type = _CONTAINERFILTER_LABELSELECTORENTRY
_LISTCONTAINERSREQUEST.fields_by_name['filter'].message_type = _CONTAINERFILTER
_CONTAINER_LABELSENTRY.containing_type = _CONTAINER
_CONTAINER_ANNOTATIONSENTRY.containing_type = _CONTAINER
_CONTAINER.fields_by_name['metadata'].message_type = _CONTAINERMETADATA
_CONTAINER.fields_by_name['image'].message_type = _IMAGESPEC
_CONTAINER.fields_by_name['state'].enum_type = _CONTAINERSTATE
_CONTAINER.fields_by_name['labels'].message_type = _CONTAINER_LABELSENTRY
_CONTAINER.fields_by_name['annotations'].message_type = _CONTAINER_ANNOTATIONSENTRY
_LISTCONTAINERSRESPONSE.fields_by_name['containers'].message_type = _CONTAINER
_CONTAINERSTATUS_LABELSENTRY.containing_type = _CONTAINERSTATUS
_CONTAINERSTATUS_ANNOTATIONSENTRY.containing_type = _CONTAINERSTATUS
_CONTAINERSTATUS.fields_by_name['metadata'].message_type = _CONTAINERMETADATA
_CONTAINERSTATUS.fields_by_name['state'].enum_type = _CONTAINERSTATE
_CONTAINERSTATUS.fields_by_name['image'].message_type = _IMAGESPEC
_CONTAINERSTATUS.fields_by_name['labels'].message_type = _CONTAINERSTATUS_LABELSENTRY
_CONTAINERSTATUS.fields_by_name['annotations'].message_type = _CONTAINERSTATUS_ANNOTATIONSENTRY
_CONTAINERSTATUS.fields_by_name['mounts'].message_type = _MOUNT
_CONTAINERSTATUSRESPONSE_INFOENTRY.containing_type = _CONTAINERSTATUSRESPONSE
_CONTAINERSTATUSRESPONSE.fields_by_name['status'].message_type = _CONTAINERSTATUS
_CONTAINERSTATUSRESPONSE.fields_by_name['info'].message_type = _CONTAINERSTATUSRESPONSE_INFOENTRY
_UPDATECONTAINERRESOURCESREQUEST.fields_by_name['linux'].message_type = _LINUXCONTAINERRESOURCES
_IMAGEFILTER.fields_by_name['image'].message_type = _IMAGESPEC
_LISTIMAGESREQUEST.fields_by_name['filter'].message_type = _IMAGEFILTER
_IMAGE.fields_by_name['uid'].message_type = _INT64VALUE
_LISTIMAGESRESPONSE.fields_by_name['images'].message_type = _IMAGE
_IMAGESTATUSREQUEST.fields_by_name['image'].message_type = _IMAGESPEC
_IMAGESTATUSRESPONSE_INFOENTRY.containing_type = _IMAGESTATUSRESPONSE
_IMAGESTATUSRESPONSE.fields_by_name['image'].message_type = _IMAGE
_IMAGESTATUSRESPONSE.fields_by_name['info'].message_type = _IMAGESTATUSRESPONSE_INFOENTRY
_PULLIMAGEREQUEST.fields_by_name['image'].message_type = _IMAGESPEC
_PULLIMAGEREQUEST.fields_by_name['auth'].message_type = _AUTHCONFIG
_PULLIMAGEREQUEST.fields_by_name['sandbox_config'].message_type = _PODSANDBOXCONFIG
_REMOVEIMAGEREQUEST.fields_by_name['image'].message_type = _IMAGESPEC
_RUNTIMECONFIG.fields_by_name['network_config'].message_type = _NETWORKCONFIG
_UPDATERUNTIMECONFIGREQUEST.fields_by_name['runtime_config'].message_type = _RUNTIMECONFIG
_RUNTIMESTATUS.fields_by_name['conditions'].message_type = _RUNTIMECONDITION
_STATUSRESPONSE_INFOENTRY.containing_type = _STATUSRESPONSE
_STATUSRESPONSE.fields_by_name['status'].message_type = _RUNTIMESTATUS
_STATUSRESPONSE.fields_by_name['info'].message_type = _STATUSRESPONSE_INFOENTRY
_FILESYSTEMUSAGE.fields_by_name['fs_id'].message_type = _FILESYSTEMIDENTIFIER
_FILESYSTEMUSAGE.fields_by_name['used_bytes'].message_type = _UINT64VALUE
_FILESYSTEMUSAGE.fields_by_name['inodes_used'].message_type = _UINT64VALUE
_IMAGEFSINFORESPONSE.fields_by_name['image_filesystems'].message_type = _FILESYSTEMUSAGE
_CONTAINERSTATSRESPONSE.fields_by_name['stats'].message_type = _CONTAINERSTATS
_LISTCONTAINERSTATSREQUEST.fields_by_name['filter'].message_type = _CONTAINERSTATSFILTER
_CONTAINERSTATSFILTER_LABELSELECTORENTRY.containing_type = _CONTAINERSTATSFILTER
_CONTAINERSTATSFILTER.fields_by_name['label_selector'].message_type = _CONTAINERSTATSFILTER_LABELSELECTORENTRY
_LISTCONTAINERSTATSRESPONSE.fields_by_name['stats'].message_type = _CONTAINERSTATS
_CONTAINERATTRIBUTES_LABELSENTRY.containing_type = _CONTAINERATTRIBUTES
_CONTAINERATTRIBUTES_ANNOTATIONSENTRY.containing_type = _CONTAINERATTRIBUTES
_CONTAINERATTRIBUTES.fields_by_name['metadata'].message_type = _CONTAINERMETADATA
_CONTAINERATTRIBUTES.fields_by_name['labels'].message_type = _CONTAINERATTRIBUTES_LABELSENTRY
_CONTAINERATTRIBUTES.fields_by_name['annotations'].message_type = _CONTAINERATTRIBUTES_ANNOTATIONSENTRY
_CONTAINERSTATS.fields_by_name['attributes'].message_type = _CONTAINERATTRIBUTES
_CONTAINERSTATS.fields_by_name['cpu'].message_type = _CPUUSAGE
_CONTAINERSTATS.fields_by_name['memory'].message_type = _MEMORYUSAGE
_CONTAINERSTATS.fields_by_name['writable_layer'].message_type = _FILESYSTEMUSAGE
_CPUUSAGE.fields_by_name['usage_core_nano_seconds'].message_type = _UINT64VALUE
_MEMORYUSAGE.fields_by_name['working_set_bytes'].message_type = _UINT64VALUE
DESCRIPTOR.message_types_by_name['VersionRequest'] = _VERSIONREQUEST
DESCRIPTOR.message_types_by_name['VersionResponse'] = _VERSIONRESPONSE
DESCRIPTOR.message_types_by_name['DNSConfig'] = _DNSCONFIG
DESCRIPTOR.message_types_by_name['PortMapping'] = _PORTMAPPING
DESCRIPTOR.message_types_by_name['Mount'] = _MOUNT
DESCRIPTOR.message_types_by_name['NamespaceOption'] = _NAMESPACEOPTION
DESCRIPTOR.message_types_by_name['Int64Value'] = _INT64VALUE
DESCRIPTOR.message_types_by_name['LinuxSandboxSecurityContext'] = _LINUXSANDBOXSECURITYCONTEXT
DESCRIPTOR.message_types_by_name['LinuxPodSandboxConfig'] = _LINUXPODSANDBOXCONFIG
DESCRIPTOR.message_types_by_name['PodSandboxMetadata'] = _PODSANDBOXMETADATA
DESCRIPTOR.message_types_by_name['PodSandboxConfig'] = _PODSANDBOXCONFIG
DESCRIPTOR.message_types_by_name['RunPodSandboxRequest'] = _RUNPODSANDBOXREQUEST
DESCRIPTOR.message_types_by_name['RunPodSandboxResponse'] = _RUNPODSANDBOXRESPONSE
DESCRIPTOR.message_types_by_name['StopPodSandboxRequest'] = _STOPPODSANDBOXREQUEST
DESCRIPTOR.message_types_by_name['StopPodSandboxResponse'] = _STOPPODSANDBOXRESPONSE
DESCRIPTOR.message_types_by_name['RemovePodSandboxRequest'] = _REMOVEPODSANDBOXREQUEST
DESCRIPTOR.message_types_by_name['RemovePodSandboxResponse'] = _REMOVEPODSANDBOXRESPONSE
DESCRIPTOR.message_types_by_name['PodSandboxStatusRequest'] = _PODSANDBOXSTATUSREQUEST
DESCRIPTOR.message_types_by_name['PodIP'] = _PODIP
DESCRIPTOR.message_types_by_name['PodSandboxNetworkStatus'] = _PODSANDBOXNETWORKSTATUS
DESCRIPTOR.message_types_by_name['Namespace'] = _NAMESPACE
DESCRIPTOR.message_types_by_name['LinuxPodSandboxStatus'] = _LINUXPODSANDBOXSTATUS
DESCRIPTOR.message_types_by_name['PodSandboxStatus'] = _PODSANDBOXSTATUS
DESCRIPTOR.message_types_by_name['PodSandboxStatusResponse'] = _PODSANDBOXSTATUSRESPONSE
DESCRIPTOR.message_types_by_name['PodSandboxStateValue'] = _PODSANDBOXSTATEVALUE
DESCRIPTOR.message_types_by_name['PodSandboxFilter'] = _PODSANDBOXFILTER
DESCRIPTOR.message_types_by_name['ListPodSandboxRequest'] = _LISTPODSANDBOXREQUEST
DESCRIPTOR.message_types_by_name['PodSandbox'] = _PODSANDBOX
DESCRIPTOR.message_types_by_name['ListPodSandboxResponse'] = _LISTPODSANDBOXRESPONSE
DESCRIPTOR.message_types_by_name['ImageSpec'] = _IMAGESPEC
DESCRIPTOR.message_types_by_name['KeyValue'] = _KEYVALUE
DESCRIPTOR.message_types_by_name['LinuxContainerResources'] = _LINUXCONTAINERRESOURCES
DESCRIPTOR.message_types_by_name['HugepageLimit'] = _HUGEPAGELIMIT
DESCRIPTOR.message_types_by_name['SELinuxOption'] = _SELINUXOPTION
DESCRIPTOR.message_types_by_name['Capability'] = _CAPABILITY
DESCRIPTOR.message_types_by_name['LinuxContainerSecurityContext'] = _LINUXCONTAINERSECURITYCONTEXT
DESCRIPTOR.message_types_by_name['LinuxContainerConfig'] = _LINUXCONTAINERCONFIG
DESCRIPTOR.message_types_by_name['WindowsContainerSecurityContext'] = _WINDOWSCONTAINERSECURITYCONTEXT
DESCRIPTOR.message_types_by_name['WindowsContainerConfig'] = _WINDOWSCONTAINERCONFIG
DESCRIPTOR.message_types_by_name['WindowsContainerResources'] = _WINDOWSCONTAINERRESOURCES
DESCRIPTOR.message_types_by_name['ContainerMetadata'] = _CONTAINERMETADATA
DESCRIPTOR.message_types_by_name['Device'] = _DEVICE
DESCRIPTOR.message_types_by_name['ContainerConfig'] = _CONTAINERCONFIG
DESCRIPTOR.message_types_by_name['CreateContainerRequest'] = _CREATECONTAINERREQUEST
DESCRIPTOR.message_types_by_name['CreateContainerResponse'] = _CREATECONTAINERRESPONSE
DESCRIPTOR.message_types_by_name['StartContainerRequest'] = _STARTCONTAINERREQUEST
DESCRIPTOR.message_types_by_name['StartContainerResponse'] = _STARTCONTAINERRESPONSE
DESCRIPTOR.message_types_by_name['StopContainerRequest'] = _STOPCONTAINERREQUEST
DESCRIPTOR.message_types_by_name['StopContainerResponse'] = _STOPCONTAINERRESPONSE
DESCRIPTOR.message_types_by_name['RemoveContainerRequest'] = _REMOVECONTAINERREQUEST
DESCRIPTOR.message_types_by_name['RemoveContainerResponse'] = _REMOVECONTAINERRESPONSE
DESCRIPTOR.message_types_by_name['ContainerStateValue'] = _CONTAINERSTATEVALUE
DESCRIPTOR.message_types_by_name['ContainerFilter'] = _CONTAINERFILTER
DESCRIPTOR.message_types_by_name['ListContainersRequest'] = _LISTCONTAINERSREQUEST
DESCRIPTOR.message_types_by_name['Container'] = _CONTAINER
DESCRIPTOR.message_types_by_name['ListContainersResponse'] = _LISTCONTAINERSRESPONSE
DESCRIPTOR.message_types_by_name['ContainerStatusRequest'] = _CONTAINERSTATUSREQUEST
DESCRIPTOR.message_types_by_name['ContainerStatus'] = _CONTAINERSTATUS
DESCRIPTOR.message_types_by_name['ContainerStatusResponse'] = _CONTAINERSTATUSRESPONSE
DESCRIPTOR.message_types_by_name['UpdateContainerResourcesRequest'] = _UPDATECONTAINERRESOURCESREQUEST
DESCRIPTOR.message_types_by_name['UpdateContainerResourcesResponse'] = _UPDATECONTAINERRESOURCESRESPONSE
DESCRIPTOR.message_types_by_name['ExecSyncRequest'] = _EXECSYNCREQUEST
DESCRIPTOR.message_types_by_name['ExecSyncResponse'] = _EXECSYNCRESPONSE
DESCRIPTOR.message_types_by_name['ExecRequest'] = _EXECREQUEST
DESCRIPTOR.message_types_by_name['ExecResponse'] = _EXECRESPONSE
DESCRIPTOR.message_types_by_name['AttachRequest'] = _ATTACHREQUEST
DESCRIPTOR.message_types_by_name['AttachResponse'] = _ATTACHRESPONSE
DESCRIPTOR.message_types_by_name['PortForwardRequest'] = _PORTFORWARDREQUEST
DESCRIPTOR.message_types_by_name['PortForwardResponse'] = _PORTFORWARDRESPONSE
DESCRIPTOR.message_types_by_name['ImageFilter'] = _IMAGEFILTER
DESCRIPTOR.message_types_by_name['ListImagesRequest'] = _LISTIMAGESREQUEST
DESCRIPTOR.message_types_by_name['Image'] = _IMAGE
DESCRIPTOR.message_types_by_name['ListImagesResponse'] = _LISTIMAGESRESPONSE
DESCRIPTOR.message_types_by_name['ImageStatusRequest'] = _IMAGESTATUSREQUEST
DESCRIPTOR.message_types_by_name['ImageStatusResponse'] = _IMAGESTATUSRESPONSE
DESCRIPTOR.message_types_by_name['AuthConfig'] = _AUTHCONFIG
DESCRIPTOR.message_types_by_name['PullImageRequest'] = _PULLIMAGEREQUEST
DESCRIPTOR.message_types_by_name['PullImageResponse'] = _PULLIMAGERESPONSE
DESCRIPTOR.message_types_by_name['RemoveImageRequest'] = _REMOVEIMAGEREQUEST
DESCRIPTOR.message_types_by_name['RemoveImageResponse'] = _REMOVEIMAGERESPONSE
DESCRIPTOR.message_types_by_name['NetworkConfig'] = _NETWORKCONFIG
DESCRIPTOR.message_types_by_name['RuntimeConfig'] = _RUNTIMECONFIG
DESCRIPTOR.message_types_by_name['UpdateRuntimeConfigRequest'] = _UPDATERUNTIMECONFIGREQUEST
DESCRIPTOR.message_types_by_name['UpdateRuntimeConfigResponse'] = _UPDATERUNTIMECONFIGRESPONSE
DESCRIPTOR.message_types_by_name['RuntimeCondition'] = _RUNTIMECONDITION
DESCRIPTOR.message_types_by_name['RuntimeStatus'] = _RUNTIMESTATUS
DESCRIPTOR.message_types_by_name['StatusRequest'] = _STATUSREQUEST
DESCRIPTOR.message_types_by_name['StatusResponse'] = _STATUSRESPONSE
DESCRIPTOR.message_types_by_name['ImageFsInfoRequest'] = _IMAGEFSINFOREQUEST
DESCRIPTOR.message_types_by_name['UInt64Value'] = _UINT64VALUE
DESCRIPTOR.message_types_by_name['FilesystemIdentifier'] = _FILESYSTEMIDENTIFIER
DESCRIPTOR.message_types_by_name['FilesystemUsage'] = _FILESYSTEMUSAGE
DESCRIPTOR.message_types_by_name['ImageFsInfoResponse'] = _IMAGEFSINFORESPONSE
DESCRIPTOR.message_types_by_name['ContainerStatsRequest'] = _CONTAINERSTATSREQUEST
DESCRIPTOR.message_types_by_name['ContainerStatsResponse'] = _CONTAINERSTATSRESPONSE
DESCRIPTOR.message_types_by_name['ListContainerStatsRequest'] = _LISTCONTAINERSTATSREQUEST
DESCRIPTOR.message_types_by_name['ContainerStatsFilter'] = _CONTAINERSTATSFILTER
DESCRIPTOR.message_types_by_name['ListContainerStatsResponse'] = _LISTCONTAINERSTATSRESPONSE
DESCRIPTOR.message_types_by_name['ContainerAttributes'] = _CONTAINERATTRIBUTES
DESCRIPTOR.message_types_by_name['ContainerStats'] = _CONTAINERSTATS
DESCRIPTOR.message_types_by_name['CpuUsage'] = _CPUUSAGE
DESCRIPTOR.message_types_by_name['MemoryUsage'] = _MEMORYUSAGE
DESCRIPTOR.message_types_by_name['ReopenContainerLogRequest'] = _REOPENCONTAINERLOGREQUEST
DESCRIPTOR.message_types_by_name['ReopenContainerLogResponse'] = _REOPENCONTAINERLOGRESPONSE
DESCRIPTOR.enum_types_by_name['Protocol'] = _PROTOCOL
DESCRIPTOR.enum_types_by_name['MountPropagation'] = _MOUNTPROPAGATION
DESCRIPTOR.enum_types_by_name['NamespaceMode'] = _NAMESPACEMODE
DESCRIPTOR.enum_types_by_name['PodSandboxState'] = _PODSANDBOXSTATE
DESCRIPTOR.enum_types_by_name['ContainerState'] = _CONTAINERSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VersionRequest = _reflection.GeneratedProtocolMessageType('VersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _VERSIONREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.VersionRequest)
  })
_sym_db.RegisterMessage(VersionRequest)

VersionResponse = _reflection.GeneratedProtocolMessageType('VersionResponse', (_message.Message,), {
  'DESCRIPTOR' : _VERSIONRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.VersionResponse)
  })
_sym_db.RegisterMessage(VersionResponse)

DNSConfig = _reflection.GeneratedProtocolMessageType('DNSConfig', (_message.Message,), {
  'DESCRIPTOR' : _DNSCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.DNSConfig)
  })
_sym_db.RegisterMessage(DNSConfig)

PortMapping = _reflection.GeneratedProtocolMessageType('PortMapping', (_message.Message,), {
  'DESCRIPTOR' : _PORTMAPPING,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PortMapping)
  })
_sym_db.RegisterMessage(PortMapping)

Mount = _reflection.GeneratedProtocolMessageType('Mount', (_message.Message,), {
  'DESCRIPTOR' : _MOUNT,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Mount)
  })
_sym_db.RegisterMessage(Mount)

NamespaceOption = _reflection.GeneratedProtocolMessageType('NamespaceOption', (_message.Message,), {
  'DESCRIPTOR' : _NAMESPACEOPTION,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.NamespaceOption)
  })
_sym_db.RegisterMessage(NamespaceOption)

Int64Value = _reflection.GeneratedProtocolMessageType('Int64Value', (_message.Message,), {
  'DESCRIPTOR' : _INT64VALUE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Int64Value)
  })
_sym_db.RegisterMessage(Int64Value)

LinuxSandboxSecurityContext = _reflection.GeneratedProtocolMessageType('LinuxSandboxSecurityContext', (_message.Message,), {
  'DESCRIPTOR' : _LINUXSANDBOXSECURITYCONTEXT,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxSandboxSecurityContext)
  })
_sym_db.RegisterMessage(LinuxSandboxSecurityContext)

LinuxPodSandboxConfig = _reflection.GeneratedProtocolMessageType('LinuxPodSandboxConfig', (_message.Message,), {

  'SysctlsEntry' : _reflection.GeneratedProtocolMessageType('SysctlsEntry', (_message.Message,), {
    'DESCRIPTOR' : _LINUXPODSANDBOXCONFIG_SYSCTLSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxPodSandboxConfig.SysctlsEntry)
    })
  ,
  'DESCRIPTOR' : _LINUXPODSANDBOXCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxPodSandboxConfig)
  })
_sym_db.RegisterMessage(LinuxPodSandboxConfig)
_sym_db.RegisterMessage(LinuxPodSandboxConfig.SysctlsEntry)

PodSandboxMetadata = _reflection.GeneratedProtocolMessageType('PodSandboxMetadata', (_message.Message,), {
  'DESCRIPTOR' : _PODSANDBOXMETADATA,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxMetadata)
  })
_sym_db.RegisterMessage(PodSandboxMetadata)

PodSandboxConfig = _reflection.GeneratedProtocolMessageType('PodSandboxConfig', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOXCONFIG_LABELSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxConfig.LabelsEntry)
    })
  ,

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOXCONFIG_ANNOTATIONSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxConfig.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _PODSANDBOXCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxConfig)
  })
_sym_db.RegisterMessage(PodSandboxConfig)
_sym_db.RegisterMessage(PodSandboxConfig.LabelsEntry)
_sym_db.RegisterMessage(PodSandboxConfig.AnnotationsEntry)

RunPodSandboxRequest = _reflection.GeneratedProtocolMessageType('RunPodSandboxRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNPODSANDBOXREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RunPodSandboxRequest)
  })
_sym_db.RegisterMessage(RunPodSandboxRequest)

RunPodSandboxResponse = _reflection.GeneratedProtocolMessageType('RunPodSandboxResponse', (_message.Message,), {
  'DESCRIPTOR' : _RUNPODSANDBOXRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RunPodSandboxResponse)
  })
_sym_db.RegisterMessage(RunPodSandboxResponse)

StopPodSandboxRequest = _reflection.GeneratedProtocolMessageType('StopPodSandboxRequest', (_message.Message,), {
  'DESCRIPTOR' : _STOPPODSANDBOXREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopPodSandboxRequest)
  })
_sym_db.RegisterMessage(StopPodSandboxRequest)

StopPodSandboxResponse = _reflection.GeneratedProtocolMessageType('StopPodSandboxResponse', (_message.Message,), {
  'DESCRIPTOR' : _STOPPODSANDBOXRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopPodSandboxResponse)
  })
_sym_db.RegisterMessage(StopPodSandboxResponse)

RemovePodSandboxRequest = _reflection.GeneratedProtocolMessageType('RemovePodSandboxRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEPODSANDBOXREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemovePodSandboxRequest)
  })
_sym_db.RegisterMessage(RemovePodSandboxRequest)

RemovePodSandboxResponse = _reflection.GeneratedProtocolMessageType('RemovePodSandboxResponse', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEPODSANDBOXRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemovePodSandboxResponse)
  })
_sym_db.RegisterMessage(RemovePodSandboxResponse)

PodSandboxStatusRequest = _reflection.GeneratedProtocolMessageType('PodSandboxStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _PODSANDBOXSTATUSREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatusRequest)
  })
_sym_db.RegisterMessage(PodSandboxStatusRequest)

PodIP = _reflection.GeneratedProtocolMessageType('PodIP', (_message.Message,), {
  'DESCRIPTOR' : _PODIP,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodIP)
  })
_sym_db.RegisterMessage(PodIP)

PodSandboxNetworkStatus = _reflection.GeneratedProtocolMessageType('PodSandboxNetworkStatus', (_message.Message,), {
  'DESCRIPTOR' : _PODSANDBOXNETWORKSTATUS,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxNetworkStatus)
  })
_sym_db.RegisterMessage(PodSandboxNetworkStatus)

Namespace = _reflection.GeneratedProtocolMessageType('Namespace', (_message.Message,), {
  'DESCRIPTOR' : _NAMESPACE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Namespace)
  })
_sym_db.RegisterMessage(Namespace)

LinuxPodSandboxStatus = _reflection.GeneratedProtocolMessageType('LinuxPodSandboxStatus', (_message.Message,), {
  'DESCRIPTOR' : _LINUXPODSANDBOXSTATUS,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxPodSandboxStatus)
  })
_sym_db.RegisterMessage(LinuxPodSandboxStatus)

PodSandboxStatus = _reflection.GeneratedProtocolMessageType('PodSandboxStatus', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOXSTATUS_LABELSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatus.LabelsEntry)
    })
  ,

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOXSTATUS_ANNOTATIONSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatus.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _PODSANDBOXSTATUS,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatus)
  })
_sym_db.RegisterMessage(PodSandboxStatus)
_sym_db.RegisterMessage(PodSandboxStatus.LabelsEntry)
_sym_db.RegisterMessage(PodSandboxStatus.AnnotationsEntry)

PodSandboxStatusResponse = _reflection.GeneratedProtocolMessageType('PodSandboxStatusResponse', (_message.Message,), {

  'InfoEntry' : _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOXSTATUSRESPONSE_INFOENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatusResponse.InfoEntry)
    })
  ,
  'DESCRIPTOR' : _PODSANDBOXSTATUSRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStatusResponse)
  })
_sym_db.RegisterMessage(PodSandboxStatusResponse)
_sym_db.RegisterMessage(PodSandboxStatusResponse.InfoEntry)

PodSandboxStateValue = _reflection.GeneratedProtocolMessageType('PodSandboxStateValue', (_message.Message,), {
  'DESCRIPTOR' : _PODSANDBOXSTATEVALUE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxStateValue)
  })
_sym_db.RegisterMessage(PodSandboxStateValue)

PodSandboxFilter = _reflection.GeneratedProtocolMessageType('PodSandboxFilter', (_message.Message,), {

  'LabelSelectorEntry' : _reflection.GeneratedProtocolMessageType('LabelSelectorEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOXFILTER_LABELSELECTORENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxFilter.LabelSelectorEntry)
    })
  ,
  'DESCRIPTOR' : _PODSANDBOXFILTER,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandboxFilter)
  })
_sym_db.RegisterMessage(PodSandboxFilter)
_sym_db.RegisterMessage(PodSandboxFilter.LabelSelectorEntry)

ListPodSandboxRequest = _reflection.GeneratedProtocolMessageType('ListPodSandboxRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTPODSANDBOXREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListPodSandboxRequest)
  })
_sym_db.RegisterMessage(ListPodSandboxRequest)

PodSandbox = _reflection.GeneratedProtocolMessageType('PodSandbox', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOX_LABELSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandbox.LabelsEntry)
    })
  ,

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _PODSANDBOX_ANNOTATIONSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandbox.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _PODSANDBOX,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PodSandbox)
  })
_sym_db.RegisterMessage(PodSandbox)
_sym_db.RegisterMessage(PodSandbox.LabelsEntry)
_sym_db.RegisterMessage(PodSandbox.AnnotationsEntry)

ListPodSandboxResponse = _reflection.GeneratedProtocolMessageType('ListPodSandboxResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTPODSANDBOXRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListPodSandboxResponse)
  })
_sym_db.RegisterMessage(ListPodSandboxResponse)

ImageSpec = _reflection.GeneratedProtocolMessageType('ImageSpec', (_message.Message,), {
  'DESCRIPTOR' : _IMAGESPEC,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageSpec)
  })
_sym_db.RegisterMessage(ImageSpec)

KeyValue = _reflection.GeneratedProtocolMessageType('KeyValue', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.KeyValue)
  })
_sym_db.RegisterMessage(KeyValue)

LinuxContainerResources = _reflection.GeneratedProtocolMessageType('LinuxContainerResources', (_message.Message,), {
  'DESCRIPTOR' : _LINUXCONTAINERRESOURCES,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxContainerResources)
  })
_sym_db.RegisterMessage(LinuxContainerResources)

HugepageLimit = _reflection.GeneratedProtocolMessageType('HugepageLimit', (_message.Message,), {
  'DESCRIPTOR' : _HUGEPAGELIMIT,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.HugepageLimit)
  })
_sym_db.RegisterMessage(HugepageLimit)

SELinuxOption = _reflection.GeneratedProtocolMessageType('SELinuxOption', (_message.Message,), {
  'DESCRIPTOR' : _SELINUXOPTION,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.SELinuxOption)
  })
_sym_db.RegisterMessage(SELinuxOption)

Capability = _reflection.GeneratedProtocolMessageType('Capability', (_message.Message,), {
  'DESCRIPTOR' : _CAPABILITY,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Capability)
  })
_sym_db.RegisterMessage(Capability)

LinuxContainerSecurityContext = _reflection.GeneratedProtocolMessageType('LinuxContainerSecurityContext', (_message.Message,), {
  'DESCRIPTOR' : _LINUXCONTAINERSECURITYCONTEXT,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxContainerSecurityContext)
  })
_sym_db.RegisterMessage(LinuxContainerSecurityContext)

LinuxContainerConfig = _reflection.GeneratedProtocolMessageType('LinuxContainerConfig', (_message.Message,), {
  'DESCRIPTOR' : _LINUXCONTAINERCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.LinuxContainerConfig)
  })
_sym_db.RegisterMessage(LinuxContainerConfig)

WindowsContainerSecurityContext = _reflection.GeneratedProtocolMessageType('WindowsContainerSecurityContext', (_message.Message,), {
  'DESCRIPTOR' : _WINDOWSCONTAINERSECURITYCONTEXT,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.WindowsContainerSecurityContext)
  })
_sym_db.RegisterMessage(WindowsContainerSecurityContext)

WindowsContainerConfig = _reflection.GeneratedProtocolMessageType('WindowsContainerConfig', (_message.Message,), {
  'DESCRIPTOR' : _WINDOWSCONTAINERCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.WindowsContainerConfig)
  })
_sym_db.RegisterMessage(WindowsContainerConfig)

WindowsContainerResources = _reflection.GeneratedProtocolMessageType('WindowsContainerResources', (_message.Message,), {
  'DESCRIPTOR' : _WINDOWSCONTAINERRESOURCES,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.WindowsContainerResources)
  })
_sym_db.RegisterMessage(WindowsContainerResources)

ContainerMetadata = _reflection.GeneratedProtocolMessageType('ContainerMetadata', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERMETADATA,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerMetadata)
  })
_sym_db.RegisterMessage(ContainerMetadata)

Device = _reflection.GeneratedProtocolMessageType('Device', (_message.Message,), {
  'DESCRIPTOR' : _DEVICE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Device)
  })
_sym_db.RegisterMessage(Device)

ContainerConfig = _reflection.GeneratedProtocolMessageType('ContainerConfig', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERCONFIG_LABELSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerConfig.LabelsEntry)
    })
  ,

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERCONFIG_ANNOTATIONSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerConfig.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINERCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerConfig)
  })
_sym_db.RegisterMessage(ContainerConfig)
_sym_db.RegisterMessage(ContainerConfig.LabelsEntry)
_sym_db.RegisterMessage(ContainerConfig.AnnotationsEntry)

CreateContainerRequest = _reflection.GeneratedProtocolMessageType('CreateContainerRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATECONTAINERREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.CreateContainerRequest)
  })
_sym_db.RegisterMessage(CreateContainerRequest)

CreateContainerResponse = _reflection.GeneratedProtocolMessageType('CreateContainerResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATECONTAINERRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.CreateContainerResponse)
  })
_sym_db.RegisterMessage(CreateContainerResponse)

StartContainerRequest = _reflection.GeneratedProtocolMessageType('StartContainerRequest', (_message.Message,), {
  'DESCRIPTOR' : _STARTCONTAINERREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StartContainerRequest)
  })
_sym_db.RegisterMessage(StartContainerRequest)

StartContainerResponse = _reflection.GeneratedProtocolMessageType('StartContainerResponse', (_message.Message,), {
  'DESCRIPTOR' : _STARTCONTAINERRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StartContainerResponse)
  })
_sym_db.RegisterMessage(StartContainerResponse)

StopContainerRequest = _reflection.GeneratedProtocolMessageType('StopContainerRequest', (_message.Message,), {
  'DESCRIPTOR' : _STOPCONTAINERREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopContainerRequest)
  })
_sym_db.RegisterMessage(StopContainerRequest)

StopContainerResponse = _reflection.GeneratedProtocolMessageType('StopContainerResponse', (_message.Message,), {
  'DESCRIPTOR' : _STOPCONTAINERRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StopContainerResponse)
  })
_sym_db.RegisterMessage(StopContainerResponse)

RemoveContainerRequest = _reflection.GeneratedProtocolMessageType('RemoveContainerRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVECONTAINERREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveContainerRequest)
  })
_sym_db.RegisterMessage(RemoveContainerRequest)

RemoveContainerResponse = _reflection.GeneratedProtocolMessageType('RemoveContainerResponse', (_message.Message,), {
  'DESCRIPTOR' : _REMOVECONTAINERRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveContainerResponse)
  })
_sym_db.RegisterMessage(RemoveContainerResponse)

ContainerStateValue = _reflection.GeneratedProtocolMessageType('ContainerStateValue', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATEVALUE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStateValue)
  })
_sym_db.RegisterMessage(ContainerStateValue)

ContainerFilter = _reflection.GeneratedProtocolMessageType('ContainerFilter', (_message.Message,), {

  'LabelSelectorEntry' : _reflection.GeneratedProtocolMessageType('LabelSelectorEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERFILTER_LABELSELECTORENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerFilter.LabelSelectorEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINERFILTER,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerFilter)
  })
_sym_db.RegisterMessage(ContainerFilter)
_sym_db.RegisterMessage(ContainerFilter.LabelSelectorEntry)

ListContainersRequest = _reflection.GeneratedProtocolMessageType('ListContainersRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTCONTAINERSREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainersRequest)
  })
_sym_db.RegisterMessage(ListContainersRequest)

Container = _reflection.GeneratedProtocolMessageType('Container', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINER_LABELSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Container.LabelsEntry)
    })
  ,

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINER_ANNOTATIONSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Container.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINER,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Container)
  })
_sym_db.RegisterMessage(Container)
_sym_db.RegisterMessage(Container.LabelsEntry)
_sym_db.RegisterMessage(Container.AnnotationsEntry)

ListContainersResponse = _reflection.GeneratedProtocolMessageType('ListContainersResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTCONTAINERSRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainersResponse)
  })
_sym_db.RegisterMessage(ListContainersResponse)

ContainerStatusRequest = _reflection.GeneratedProtocolMessageType('ContainerStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATUSREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatusRequest)
  })
_sym_db.RegisterMessage(ContainerStatusRequest)

ContainerStatus = _reflection.GeneratedProtocolMessageType('ContainerStatus', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERSTATUS_LABELSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatus.LabelsEntry)
    })
  ,

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERSTATUS_ANNOTATIONSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatus.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINERSTATUS,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatus)
  })
_sym_db.RegisterMessage(ContainerStatus)
_sym_db.RegisterMessage(ContainerStatus.LabelsEntry)
_sym_db.RegisterMessage(ContainerStatus.AnnotationsEntry)

ContainerStatusResponse = _reflection.GeneratedProtocolMessageType('ContainerStatusResponse', (_message.Message,), {

  'InfoEntry' : _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERSTATUSRESPONSE_INFOENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatusResponse.InfoEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINERSTATUSRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatusResponse)
  })
_sym_db.RegisterMessage(ContainerStatusResponse)
_sym_db.RegisterMessage(ContainerStatusResponse.InfoEntry)

UpdateContainerResourcesRequest = _reflection.GeneratedProtocolMessageType('UpdateContainerResourcesRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATECONTAINERRESOURCESREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateContainerResourcesRequest)
  })
_sym_db.RegisterMessage(UpdateContainerResourcesRequest)

UpdateContainerResourcesResponse = _reflection.GeneratedProtocolMessageType('UpdateContainerResourcesResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATECONTAINERRESOURCESRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateContainerResourcesResponse)
  })
_sym_db.RegisterMessage(UpdateContainerResourcesResponse)

ExecSyncRequest = _reflection.GeneratedProtocolMessageType('ExecSyncRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXECSYNCREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecSyncRequest)
  })
_sym_db.RegisterMessage(ExecSyncRequest)

ExecSyncResponse = _reflection.GeneratedProtocolMessageType('ExecSyncResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXECSYNCRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecSyncResponse)
  })
_sym_db.RegisterMessage(ExecSyncResponse)

ExecRequest = _reflection.GeneratedProtocolMessageType('ExecRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXECREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecRequest)
  })
_sym_db.RegisterMessage(ExecRequest)

ExecResponse = _reflection.GeneratedProtocolMessageType('ExecResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXECRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ExecResponse)
  })
_sym_db.RegisterMessage(ExecResponse)

AttachRequest = _reflection.GeneratedProtocolMessageType('AttachRequest', (_message.Message,), {
  'DESCRIPTOR' : _ATTACHREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.AttachRequest)
  })
_sym_db.RegisterMessage(AttachRequest)

AttachResponse = _reflection.GeneratedProtocolMessageType('AttachResponse', (_message.Message,), {
  'DESCRIPTOR' : _ATTACHRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.AttachResponse)
  })
_sym_db.RegisterMessage(AttachResponse)

PortForwardRequest = _reflection.GeneratedProtocolMessageType('PortForwardRequest', (_message.Message,), {
  'DESCRIPTOR' : _PORTFORWARDREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PortForwardRequest)
  })
_sym_db.RegisterMessage(PortForwardRequest)

PortForwardResponse = _reflection.GeneratedProtocolMessageType('PortForwardResponse', (_message.Message,), {
  'DESCRIPTOR' : _PORTFORWARDRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PortForwardResponse)
  })
_sym_db.RegisterMessage(PortForwardResponse)

ImageFilter = _reflection.GeneratedProtocolMessageType('ImageFilter', (_message.Message,), {
  'DESCRIPTOR' : _IMAGEFILTER,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageFilter)
  })
_sym_db.RegisterMessage(ImageFilter)

ListImagesRequest = _reflection.GeneratedProtocolMessageType('ListImagesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTIMAGESREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListImagesRequest)
  })
_sym_db.RegisterMessage(ListImagesRequest)

Image = _reflection.GeneratedProtocolMessageType('Image', (_message.Message,), {
  'DESCRIPTOR' : _IMAGE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.Image)
  })
_sym_db.RegisterMessage(Image)

ListImagesResponse = _reflection.GeneratedProtocolMessageType('ListImagesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTIMAGESRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListImagesResponse)
  })
_sym_db.RegisterMessage(ListImagesResponse)

ImageStatusRequest = _reflection.GeneratedProtocolMessageType('ImageStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _IMAGESTATUSREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageStatusRequest)
  })
_sym_db.RegisterMessage(ImageStatusRequest)

ImageStatusResponse = _reflection.GeneratedProtocolMessageType('ImageStatusResponse', (_message.Message,), {

  'InfoEntry' : _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), {
    'DESCRIPTOR' : _IMAGESTATUSRESPONSE_INFOENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageStatusResponse.InfoEntry)
    })
  ,
  'DESCRIPTOR' : _IMAGESTATUSRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageStatusResponse)
  })
_sym_db.RegisterMessage(ImageStatusResponse)
_sym_db.RegisterMessage(ImageStatusResponse.InfoEntry)

AuthConfig = _reflection.GeneratedProtocolMessageType('AuthConfig', (_message.Message,), {
  'DESCRIPTOR' : _AUTHCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.AuthConfig)
  })
_sym_db.RegisterMessage(AuthConfig)

PullImageRequest = _reflection.GeneratedProtocolMessageType('PullImageRequest', (_message.Message,), {
  'DESCRIPTOR' : _PULLIMAGEREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PullImageRequest)
  })
_sym_db.RegisterMessage(PullImageRequest)

PullImageResponse = _reflection.GeneratedProtocolMessageType('PullImageResponse', (_message.Message,), {
  'DESCRIPTOR' : _PULLIMAGERESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.PullImageResponse)
  })
_sym_db.RegisterMessage(PullImageResponse)

RemoveImageRequest = _reflection.GeneratedProtocolMessageType('RemoveImageRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEIMAGEREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveImageRequest)
  })
_sym_db.RegisterMessage(RemoveImageRequest)

RemoveImageResponse = _reflection.GeneratedProtocolMessageType('RemoveImageResponse', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEIMAGERESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RemoveImageResponse)
  })
_sym_db.RegisterMessage(RemoveImageResponse)

NetworkConfig = _reflection.GeneratedProtocolMessageType('NetworkConfig', (_message.Message,), {
  'DESCRIPTOR' : _NETWORKCONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.NetworkConfig)
  })
_sym_db.RegisterMessage(NetworkConfig)

RuntimeConfig = _reflection.GeneratedProtocolMessageType('RuntimeConfig', (_message.Message,), {
  'DESCRIPTOR' : _RUNTIMECONFIG,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RuntimeConfig)
  })
_sym_db.RegisterMessage(RuntimeConfig)

UpdateRuntimeConfigRequest = _reflection.GeneratedProtocolMessageType('UpdateRuntimeConfigRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATERUNTIMECONFIGREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateRuntimeConfigRequest)
  })
_sym_db.RegisterMessage(UpdateRuntimeConfigRequest)

UpdateRuntimeConfigResponse = _reflection.GeneratedProtocolMessageType('UpdateRuntimeConfigResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATERUNTIMECONFIGRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.UpdateRuntimeConfigResponse)
  })
_sym_db.RegisterMessage(UpdateRuntimeConfigResponse)

RuntimeCondition = _reflection.GeneratedProtocolMessageType('RuntimeCondition', (_message.Message,), {
  'DESCRIPTOR' : _RUNTIMECONDITION,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RuntimeCondition)
  })
_sym_db.RegisterMessage(RuntimeCondition)

RuntimeStatus = _reflection.GeneratedProtocolMessageType('RuntimeStatus', (_message.Message,), {
  'DESCRIPTOR' : _RUNTIMESTATUS,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.RuntimeStatus)
  })
_sym_db.RegisterMessage(RuntimeStatus)

StatusRequest = _reflection.GeneratedProtocolMessageType('StatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _STATUSREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StatusRequest)
  })
_sym_db.RegisterMessage(StatusRequest)

StatusResponse = _reflection.GeneratedProtocolMessageType('StatusResponse', (_message.Message,), {

  'InfoEntry' : _reflection.GeneratedProtocolMessageType('InfoEntry', (_message.Message,), {
    'DESCRIPTOR' : _STATUSRESPONSE_INFOENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StatusResponse.InfoEntry)
    })
  ,
  'DESCRIPTOR' : _STATUSRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.StatusResponse)
  })
_sym_db.RegisterMessage(StatusResponse)
_sym_db.RegisterMessage(StatusResponse.InfoEntry)

ImageFsInfoRequest = _reflection.GeneratedProtocolMessageType('ImageFsInfoRequest', (_message.Message,), {
  'DESCRIPTOR' : _IMAGEFSINFOREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageFsInfoRequest)
  })
_sym_db.RegisterMessage(ImageFsInfoRequest)

UInt64Value = _reflection.GeneratedProtocolMessageType('UInt64Value', (_message.Message,), {
  'DESCRIPTOR' : _UINT64VALUE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.UInt64Value)
  })
_sym_db.RegisterMessage(UInt64Value)

FilesystemIdentifier = _reflection.GeneratedProtocolMessageType('FilesystemIdentifier', (_message.Message,), {
  'DESCRIPTOR' : _FILESYSTEMIDENTIFIER,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.FilesystemIdentifier)
  })
_sym_db.RegisterMessage(FilesystemIdentifier)

FilesystemUsage = _reflection.GeneratedProtocolMessageType('FilesystemUsage', (_message.Message,), {
  'DESCRIPTOR' : _FILESYSTEMUSAGE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.FilesystemUsage)
  })
_sym_db.RegisterMessage(FilesystemUsage)

ImageFsInfoResponse = _reflection.GeneratedProtocolMessageType('ImageFsInfoResponse', (_message.Message,), {
  'DESCRIPTOR' : _IMAGEFSINFORESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ImageFsInfoResponse)
  })
_sym_db.RegisterMessage(ImageFsInfoResponse)

ContainerStatsRequest = _reflection.GeneratedProtocolMessageType('ContainerStatsRequest', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATSREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatsRequest)
  })
_sym_db.RegisterMessage(ContainerStatsRequest)

ContainerStatsResponse = _reflection.GeneratedProtocolMessageType('ContainerStatsResponse', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATSRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatsResponse)
  })
_sym_db.RegisterMessage(ContainerStatsResponse)

ListContainerStatsRequest = _reflection.GeneratedProtocolMessageType('ListContainerStatsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTCONTAINERSTATSREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainerStatsRequest)
  })
_sym_db.RegisterMessage(ListContainerStatsRequest)

ContainerStatsFilter = _reflection.GeneratedProtocolMessageType('ContainerStatsFilter', (_message.Message,), {

  'LabelSelectorEntry' : _reflection.GeneratedProtocolMessageType('LabelSelectorEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERSTATSFILTER_LABELSELECTORENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatsFilter.LabelSelectorEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINERSTATSFILTER,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStatsFilter)
  })
_sym_db.RegisterMessage(ContainerStatsFilter)
_sym_db.RegisterMessage(ContainerStatsFilter.LabelSelectorEntry)

ListContainerStatsResponse = _reflection.GeneratedProtocolMessageType('ListContainerStatsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTCONTAINERSTATSRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ListContainerStatsResponse)
  })
_sym_db.RegisterMessage(ListContainerStatsResponse)

ContainerAttributes = _reflection.GeneratedProtocolMessageType('ContainerAttributes', (_message.Message,), {

  'LabelsEntry' : _reflection.GeneratedProtocolMessageType('LabelsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERATTRIBUTES_LABELSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerAttributes.LabelsEntry)
    })
  ,

  'AnnotationsEntry' : _reflection.GeneratedProtocolMessageType('AnnotationsEntry', (_message.Message,), {
    'DESCRIPTOR' : _CONTAINERATTRIBUTES_ANNOTATIONSENTRY,
    '__module__' : 'zun.criapi.api_pb2'
    # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerAttributes.AnnotationsEntry)
    })
  ,
  'DESCRIPTOR' : _CONTAINERATTRIBUTES,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerAttributes)
  })
_sym_db.RegisterMessage(ContainerAttributes)
_sym_db.RegisterMessage(ContainerAttributes.LabelsEntry)
_sym_db.RegisterMessage(ContainerAttributes.AnnotationsEntry)

ContainerStats = _reflection.GeneratedProtocolMessageType('ContainerStats', (_message.Message,), {
  'DESCRIPTOR' : _CONTAINERSTATS,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ContainerStats)
  })
_sym_db.RegisterMessage(ContainerStats)

CpuUsage = _reflection.GeneratedProtocolMessageType('CpuUsage', (_message.Message,), {
  'DESCRIPTOR' : _CPUUSAGE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.CpuUsage)
  })
_sym_db.RegisterMessage(CpuUsage)

MemoryUsage = _reflection.GeneratedProtocolMessageType('MemoryUsage', (_message.Message,), {
  'DESCRIPTOR' : _MEMORYUSAGE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.MemoryUsage)
  })
_sym_db.RegisterMessage(MemoryUsage)

ReopenContainerLogRequest = _reflection.GeneratedProtocolMessageType('ReopenContainerLogRequest', (_message.Message,), {
  'DESCRIPTOR' : _REOPENCONTAINERLOGREQUEST,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ReopenContainerLogRequest)
  })
_sym_db.RegisterMessage(ReopenContainerLogRequest)

ReopenContainerLogResponse = _reflection.GeneratedProtocolMessageType('ReopenContainerLogResponse', (_message.Message,), {
  'DESCRIPTOR' : _REOPENCONTAINERLOGRESPONSE,
  '__module__' : 'zun.criapi.api_pb2'
  # @@protoc_insertion_point(class_scope:runtime.v1alpha2.ReopenContainerLogResponse)
  })
_sym_db.RegisterMessage(ReopenContainerLogResponse)


DESCRIPTOR._options = None
_LINUXPODSANDBOXCONFIG_SYSCTLSENTRY._options = None
_PODSANDBOXCONFIG_LABELSENTRY._options = None
_PODSANDBOXCONFIG_ANNOTATIONSENTRY._options = None
_PODSANDBOXSTATUS_LABELSENTRY._options = None
_PODSANDBOXSTATUS_ANNOTATIONSENTRY._options = None
_PODSANDBOXSTATUSRESPONSE_INFOENTRY._options = None
_PODSANDBOXFILTER_LABELSELECTORENTRY._options = None
_PODSANDBOX_LABELSENTRY._options = None
_PODSANDBOX_ANNOTATIONSENTRY._options = None
_CONTAINERCONFIG_LABELSENTRY._options = None
_CONTAINERCONFIG_ANNOTATIONSENTRY._options = None
_CONTAINERFILTER_LABELSELECTORENTRY._options = None
_CONTAINER_LABELSENTRY._options = None
_CONTAINER_ANNOTATIONSENTRY._options = None
_CONTAINERSTATUS_LABELSENTRY._options = None
_CONTAINERSTATUS_ANNOTATIONSENTRY._options = None
_CONTAINERSTATUSRESPONSE_INFOENTRY._options = None
_IMAGESTATUSRESPONSE_INFOENTRY._options = None
_STATUSRESPONSE_INFOENTRY._options = None
_CONTAINERSTATSFILTER_LABELSELECTORENTRY._options = None
_CONTAINERATTRIBUTES_LABELSENTRY._options = None
_CONTAINERATTRIBUTES_ANNOTATIONSENTRY._options = None

_RUNTIMESERVICE = _descriptor.ServiceDescriptor(
  name='RuntimeService',
  full_name='runtime.v1alpha2.RuntimeService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=13103,
  serialized_end=15344,
  methods=[
  _descriptor.MethodDescriptor(
    name='Version',
    full_name='runtime.v1alpha2.RuntimeService.Version',
    index=0,
    containing_service=None,
    input_type=_VERSIONREQUEST,
    output_type=_VERSIONRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RunPodSandbox',
    full_name='runtime.v1alpha2.RuntimeService.RunPodSandbox',
    index=1,
    containing_service=None,
    input_type=_RUNPODSANDBOXREQUEST,
    output_type=_RUNPODSANDBOXRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='StopPodSandbox',
    full_name='runtime.v1alpha2.RuntimeService.StopPodSandbox',
    index=2,
    containing_service=None,
    input_type=_STOPPODSANDBOXREQUEST,
    output_type=_STOPPODSANDBOXRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RemovePodSandbox',
    full_name='runtime.v1alpha2.RuntimeService.RemovePodSandbox',
    index=3,
    containing_service=None,
    input_type=_REMOVEPODSANDBOXREQUEST,
    output_type=_REMOVEPODSANDBOXRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PodSandboxStatus',
    full_name='runtime.v1alpha2.RuntimeService.PodSandboxStatus',
    index=4,
    containing_service=None,
    input_type=_PODSANDBOXSTATUSREQUEST,
    output_type=_PODSANDBOXSTATUSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListPodSandbox',
    full_name='runtime.v1alpha2.RuntimeService.ListPodSandbox',
    index=5,
    containing_service=None,
    input_type=_LISTPODSANDBOXREQUEST,
    output_type=_LISTPODSANDBOXRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='CreateContainer',
    full_name='runtime.v1alpha2.RuntimeService.CreateContainer',
    index=6,
    containing_service=None,
    input_type=_CREATECONTAINERREQUEST,
    output_type=_CREATECONTAINERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='StartContainer',
    full_name='runtime.v1alpha2.RuntimeService.StartContainer',
    index=7,
    containing_service=None,
    input_type=_STARTCONTAINERREQUEST,
    output_type=_STARTCONTAINERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='StopContainer',
    full_name='runtime.v1alpha2.RuntimeService.StopContainer',
    index=8,
    containing_service=None,
    input_type=_STOPCONTAINERREQUEST,
    output_type=_STOPCONTAINERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveContainer',
    full_name='runtime.v1alpha2.RuntimeService.RemoveContainer',
    index=9,
    containing_service=None,
    input_type=_REMOVECONTAINERREQUEST,
    output_type=_REMOVECONTAINERRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListContainers',
    full_name='runtime.v1alpha2.RuntimeService.ListContainers',
    index=10,
    containing_service=None,
    input_type=_LISTCONTAINERSREQUEST,
    output_type=_LISTCONTAINERSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ContainerStatus',
    full_name='runtime.v1alpha2.RuntimeService.ContainerStatus',
    index=11,
    containing_service=None,
    input_type=_CONTAINERSTATUSREQUEST,
    output_type=_CONTAINERSTATUSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateContainerResources',
    full_name='runtime.v1alpha2.RuntimeService.UpdateContainerResources',
    index=12,
    containing_service=None,
    input_type=_UPDATECONTAINERRESOURCESREQUEST,
    output_type=_UPDATECONTAINERRESOURCESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ReopenContainerLog',
    full_name='runtime.v1alpha2.RuntimeService.ReopenContainerLog',
    index=13,
    containing_service=None,
    input_type=_REOPENCONTAINERLOGREQUEST,
    output_type=_REOPENCONTAINERLOGRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ExecSync',
    full_name='runtime.v1alpha2.RuntimeService.ExecSync',
    index=14,
    containing_service=None,
    input_type=_EXECSYNCREQUEST,
    output_type=_EXECSYNCRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Exec',
    full_name='runtime.v1alpha2.RuntimeService.Exec',
    index=15,
    containing_service=None,
    input_type=_EXECREQUEST,
    output_type=_EXECRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Attach',
    full_name='runtime.v1alpha2.RuntimeService.Attach',
    index=16,
    containing_service=None,
    input_type=_ATTACHREQUEST,
    output_type=_ATTACHRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PortForward',
    full_name='runtime.v1alpha2.RuntimeService.PortForward',
    index=17,
    containing_service=None,
    input_type=_PORTFORWARDREQUEST,
    output_type=_PORTFORWARDRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ContainerStats',
    full_name='runtime.v1alpha2.RuntimeService.ContainerStats',
    index=18,
    containing_service=None,
    input_type=_CONTAINERSTATSREQUEST,
    output_type=_CONTAINERSTATSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ListContainerStats',
    full_name='runtime.v1alpha2.RuntimeService.ListContainerStats',
    index=19,
    containing_service=None,
    input_type=_LISTCONTAINERSTATSREQUEST,
    output_type=_LISTCONTAINERSTATSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateRuntimeConfig',
    full_name='runtime.v1alpha2.RuntimeService.UpdateRuntimeConfig',
    index=20,
    containing_service=None,
    input_type=_UPDATERUNTIMECONFIGREQUEST,
    output_type=_UPDATERUNTIMECONFIGRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='Status',
    full_name='runtime.v1alpha2.RuntimeService.Status',
    index=21,
    containing_service=None,
    input_type=_STATUSREQUEST,
    output_type=_STATUSRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_RUNTIMESERVICE)

DESCRIPTOR.services_by_name['RuntimeService'] = _RUNTIMESERVICE


_IMAGESERVICE = _descriptor.ServiceDescriptor(
  name='ImageService',
  full_name='runtime.v1alpha2.ImageService',
  file=DESCRIPTOR,
  index=1,
  serialized_options=None,
  serialized_start=15347,
  serialized_end=15822,
  methods=[
  _descriptor.MethodDescriptor(
    name='ListImages',
    full_name='runtime.v1alpha2.ImageService.ListImages',
    index=0,
    containing_service=None,
    input_type=_LISTIMAGESREQUEST,
    output_type=_LISTIMAGESRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ImageStatus',
    full_name='runtime.v1alpha2.ImageService.ImageStatus',
    index=1,
    containing_service=None,
    input_type=_IMAGESTATUSREQUEST,
    output_type=_IMAGESTATUSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='PullImage',
    full_name='runtime.v1alpha2.ImageService.PullImage',
    index=2,
    containing_service=None,
    input_type=_PULLIMAGEREQUEST,
    output_type=_PULLIMAGERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RemoveImage',
    full_name='runtime.v1alpha2.ImageService.RemoveImage',
    index=3,
    containing_service=None,
    input_type=_REMOVEIMAGEREQUEST,
    output_type=_REMOVEIMAGERESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='ImageFsInfo',
    full_name='runtime.v1alpha2.ImageService.ImageFsInfo',
    index=4,
    containing_service=None,
    input_type=_IMAGEFSINFOREQUEST,
    output_type=_IMAGEFSINFORESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_IMAGESERVICE)

DESCRIPTOR.services_by_name['ImageService'] = _IMAGESERVICE

# @@protoc_insertion_point(module_scope)
